@@ -28,15 +28,25 @@
 extern "C" {
 #endif
 #include "linphonecore.h"
+#include "linphonefriend.h"
 #include "linphone_tunnel.h"
 #include "linphonecore_utils.h"
 #include "sal.h"
+#include "sipsetup.h"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include "mediastreamer2/ice.h"
 #include "mediastreamer2/mediastream.h"
 #include "mediastreamer2/msconference.h"
+#ifdef BUILD_UPNP
+#include "upnp.h"
+#endif //BUILD_UPNP
+
+#ifdef MSG_STORAGE_ENABLED
+#include "sqlite3.h"
+#endif
 
 #ifndef LIBLINPHONE_VERSION
 #define LIBLINPHONE_VERSION LINPHONE_VERSION
@@ -68,19 +78,82 @@ struct _LinphoneCallParams{
 	LinphoneCall *referer; /*in case this call creation is consecutive to an incoming transfer, this points to the original call */
 	int audio_bw; /* bandwidth limit for audio stream */
 	LinphoneMediaEncryption media_encryption;
+	PayloadType *audio_codec; /*audio codec currently in use */
+	PayloadType *video_codec; /*video codec currently in use */
+	int down_bw;
+	int up_bw;
+	int down_ptime;
+	int up_ptime;
+	char *record_file;
+	SalCustomHeader *custom_headers;
 	bool_t has_video;
 	bool_t real_early_media; /*send real media even during early media (for outgoing calls)*/
 	bool_t in_conference; /*in conference mode */
 	bool_t pad;
-	
+	bool_t low_bandwidth;
 };
 
+struct _LinphoneCallLog{
+	struct _LinphoneCore *lc;
+	LinphoneCallDir dir; /**< The direction of the call*/
+	LinphoneCallStatus status; /**< The status of the call*/
+	LinphoneAddress *from; /**<Originator of the call as a LinphoneAddress object*/
+	LinphoneAddress *to; /**<Destination of the call as a LinphoneAddress object*/
+	char start_date[128]; /**<Human readable string containing the start date*/
+	int duration; /**<Duration of the call in seconds*/
+	char *refkey;
+	void *user_pointer;
+	rtp_stats_t local_stats;
+	rtp_stats_t remote_stats;
+	float quality;
+	time_t start_date_time; /**Start date of the call in seconds as expressed in a time_t */
+	char* call_id; /**unique id of a call*/
+	bool_t video_enabled;
+};
+
+typedef struct _CallCallbackObj
+{
+	LinphoneCallCbFunc _func;
+	void * _user_data;
+}CallCallbackObj;
+
+static const int linphone_call_magic=0x3343;
+
+typedef enum _LinphoneChatMessageDir{
+	LinphoneChatMessageIncoming,
+	LinphoneChatMessageOutgoing
+} LinphoneChatMessageDir;
+
+struct _LinphoneChatMessage {
+	LinphoneChatRoom* chat_room;
+	LinphoneChatMessageDir dir;
+	char* message;
+	LinphoneChatMessageStateChangeCb cb;
+	void* cb_ud;
+	void* message_userdata;
+	char* external_body_url;
+	LinphoneAddress *from;
+	LinphoneAddress *to;
+	time_t time;
+	SalCustomHeader *custom_headers;
+	LinphoneChatMessageState state;
+	bool_t is_read;
+};
+
+typedef struct StunCandidate{
+	char addr[64];
+	int port;
+}StunCandidate;
+
+
 struct _LinphoneCall
 {
+	int magic; /*used to distinguish from proxy config*/
 	struct _LinphoneCore *core;
 	SalMediaDescription *localdesc;
 	SalMediaDescription *resultdesc;
 	LinphoneCallDir dir;
+	LinphoneCall *referer; /*when this call is the result of a transfer, referer is set to the original call that caused the transfer*/
 	struct _RtpProfile *audio_profile;
 	struct _RtpProfile *video_profile;
 	struct _LinphoneCallLog *log;
@@ -90,33 +163,55 @@ struct _LinphoneCall
 	time_t start_time; /*time at which the call was initiated*/
 	time_t media_start_time; /*time at which it was accepted, media streams established*/
 	LinphoneCallState	state;
+	LinphoneCallState transfer_state; /*idle if no transfer*/
 	LinphoneReason reason;
+	LinphoneProxyConfig *dest_proxy;
 	int refcnt;
 	void * user_pointer;
 	int audio_port;
 	int video_port;
+	StunCandidate ac,vc; /*audio video ip/port discovered by STUN*/
 	struct _AudioStream *audiostream;  /**/
 	struct _VideoStream *videostream;
 	MSAudioEndpoint *endpoint; /*used for conferencing*/
 	char *refer_to;
 	LinphoneCallParams params;
 	LinphoneCallParams current_params;
+	LinphoneCallParams remote_params;
 	int up_bw; /*upload bandwidth setting at the time the call is started. Used to detect if it changes during a call */
 	int audio_bw;	/*upload bandwidth used by audio */
+	OrtpEvQueue *audiostream_app_evq;
+	char *auth_token;
+	OrtpEvQueue *videostream_app_evq;
+	CallCallbackObj nextVideoFrameDecoded;
+	LinphoneCallStats stats[2];
+#ifdef BUILD_UPNP
+	UpnpSession *upnp_session;
+#endif //BUILD_UPNP
+	IceSession *ice_session;
+	LinphoneChatMessage* pending_message;
+	int ping_time;
+	unsigned int remote_session_id;
+	unsigned int remote_session_ver;
 	bool_t refer_pending;
 	bool_t media_pending;
 	bool_t audio_muted;
 	bool_t camera_active;
+	
 	bool_t all_muted; /*this flag is set during early medias*/
 	bool_t playing_ringbacktone;
 	bool_t owns_call_log;
 	bool_t ringing_beep; /* whether this call is ringing through an already existent current call*/
-	OrtpEvQueue *audiostream_app_evq;
-	char *auth_token;
-	OrtpEvQueue *videostream_app_evq;
+	
 	bool_t videostream_encrypted;
 	bool_t audiostream_encrypted;
 	bool_t auth_token_verified;
+	bool_t defer_update;
+	
+	bool_t was_automatically_paused;
+	bool_t ping_replied;
+	bool_t record_active;
+	bool_t paused_by_app;
 };
 
 
@@ -128,6 +223,7 @@ void linphone_call_set_state(LinphoneCall *call, LinphoneCallState cstate, const
 LinphoneCallLog * linphone_call_log_new(LinphoneCall *call, LinphoneAddress *local, LinphoneAddress * remote);
 void linphone_call_log_completed(LinphoneCall *call);
 void linphone_call_log_destroy(LinphoneCallLog *cl);
+void linphone_call_set_transfer_state(LinphoneCall* call, LinphoneCallState state);
 
 void linphone_auth_info_write_config(struct _LpConfig *config, LinphoneAuthInfo *obj, int pos);
 
@@ -148,10 +244,7 @@ int parse_hostname_to_addr(const char *server, struct sockaddr_storage *ss, sock
 int set_lock_file();
 int get_lock_file();
 int remove_lock_file();
-int do_registration(LinphoneCore *lc, bool_t doit);
-void check_for_registration(LinphoneCore *lc);
 void check_sound_device(LinphoneCore *lc);
-void linphone_core_verify_codecs(LinphoneCore *lc);
 void linphone_core_get_local_ip(LinphoneCore *lc, const char *to, char *result);
 bool_t host_has_ipv6_network();
 bool_t lp_spawn_command_line_sync(const char *command, char **result,int *command_ret);
@@ -188,7 +281,7 @@ SalPresenceStatus linphone_online_status_to_sal(LinphoneOnlineStatus os);
 void linphone_process_authentication(LinphoneCore* lc, SalOp *op);
 void linphone_authentication_ok(LinphoneCore *lc, SalOp *op);
 void linphone_subscription_new(LinphoneCore *lc, SalOp *op, const char *from);
-void linphone_notify_recv(LinphoneCore *lc, SalOp *op, SalSubscribeState ss, SalPresenceStatus status);
+void linphone_notify_recv(LinphoneCore *lc, SalOp *op, SalSubscribeStatus ss, SalPresenceStatus status);
 void linphone_proxy_config_process_authentication_failure(LinphoneCore *lc, SalOp *op);
 
 void linphone_subscription_answered(LinphoneCore *lc, SalOp *op);
@@ -198,7 +291,14 @@ MSList *linphone_find_friend(MSList *fl, const LinphoneAddress *fri, LinphoneFri
 
 void linphone_core_update_allocated_audio_bandwidth(LinphoneCore *lc);
 void linphone_core_update_allocated_audio_bandwidth_in_call(LinphoneCall *call, const PayloadType *pt);
-void linphone_core_run_stun_tests(LinphoneCore *lc, LinphoneCall *call);
+
+int linphone_core_run_stun_tests(LinphoneCore *lc, LinphoneCall *call);
+void linphone_core_adapt_to_network(LinphoneCore *lc, int ping_time_ms, LinphoneCallParams *params);
+int linphone_core_gather_ice_candidates(LinphoneCore *lc, LinphoneCall *call);
+void linphone_core_update_ice_state_in_call_stats(LinphoneCall *call);
+void linphone_core_update_local_media_description_from_ice(SalMediaDescription *desc, IceSession *session);
+void linphone_core_update_ice_from_remote_media_description(LinphoneCall *call, const SalMediaDescription *md);
+bool_t linphone_core_media_description_contains_video_stream(const SalMediaDescription *md);
 
 void linphone_core_send_initial_subscribes(LinphoneCore *lc);
 void linphone_core_write_friends_config(LinphoneCore* lc);
@@ -216,13 +316,25 @@ void linphone_proxy_config_write_to_config_file(struct _LpConfig* config,Linphon
 
 int linphone_proxy_config_normalize_number(LinphoneProxyConfig *cfg, const char *username, char *result, size_t result_len);
 
-void linphone_core_text_received(LinphoneCore *lc, const char *from, const char *msg);
+void linphone_core_message_received(LinphoneCore *lc, SalOp *op, const SalMessage *msg);
 
 void linphone_core_play_tone(LinphoneCore *lc);
 
+void linphone_call_init_stats(LinphoneCallStats *stats, int type);
+
+void linphone_call_init_audio_stream(LinphoneCall *call);
+void linphone_call_init_video_stream(LinphoneCall *call);
 void linphone_call_init_media_streams(LinphoneCall *call);
 void linphone_call_start_media_streams(LinphoneCall *call, bool_t all_inputs_muted, bool_t send_ringbacktone);
+void linphone_call_start_media_streams_for_ice_gathering(LinphoneCall *call);
+void linphone_call_stop_audio_stream(LinphoneCall *call);
+void linphone_call_stop_video_stream(LinphoneCall *call);
 void linphone_call_stop_media_streams(LinphoneCall *call);
+void linphone_call_delete_ice_session(LinphoneCall *call);
+void linphone_call_delete_upnp_session(LinphoneCall *call);
+void linphone_call_stop_media_streams_for_ice_gathering(LinphoneCall *call);
+void linphone_call_update_crypto_parameters(LinphoneCall *call, SalMediaDescription *old_md, SalMediaDescription *new_md);
+void linphone_call_update_remote_session_id_and_ver(LinphoneCall *call);
 
 const char * linphone_core_get_identity(LinphoneCore *lc);
 const char * linphone_core_get_route(LinphoneCore *lc);
@@ -230,20 +342,35 @@ void linphone_core_start_waiting(LinphoneCore *lc, const char *purpose);
 void linphone_core_update_progress(LinphoneCore *lc, const char *purpose, float progresses);
 void linphone_core_stop_waiting(LinphoneCore *lc);
 
-int linphone_core_start_invite(LinphoneCore *lc, LinphoneCall *call, LinphoneProxyConfig *dest_proxy);
+int linphone_core_proceed_with_invite_if_ready(LinphoneCore *lc, LinphoneCall *call, LinphoneProxyConfig *dest_proxy);
+int linphone_core_start_invite(LinphoneCore *lc, LinphoneCall *call);
+int linphone_core_start_update_call(LinphoneCore *lc, LinphoneCall *call);
+int linphone_core_start_accept_call_update(LinphoneCore *lc, LinphoneCall *call);
 void linphone_core_start_refered_call(LinphoneCore *lc, LinphoneCall *call);
+void linphone_core_notify_incoming_call(LinphoneCore *lc, LinphoneCall *call);
+bool_t linphone_core_incompatible_security(LinphoneCore *lc, SalMediaDescription *md);
 extern SalCallbacks linphone_sal_callbacks;
 void linphone_proxy_config_set_error(LinphoneProxyConfig *cfg, LinphoneReason error);
 bool_t linphone_core_rtcp_enabled(const LinphoneCore *lc);
 
+LinphoneCall * is_a_linphone_call(void *user_pointer);
+LinphoneProxyConfig * is_a_linphone_proxy_config(void *user_pointer);
+
+static const int linphone_proxy_config_magic=0x7979;
+
+/*chat*/	
+void linphone_chat_message_destroy(LinphoneChatMessage* msg);
+/**/	
 
 struct _LinphoneProxyConfig
 {
+	int magic;
 	struct _LinphoneCore *lc;
 	char *reg_proxy;
 	char *reg_identity;
 	char *reg_route;
 	char *realm;
+	char *contact_params;
 	int expires;
 	int reg_time;
 	SalOp *op;
@@ -278,10 +405,12 @@ struct _LinphoneChatRoom{
 	struct _LinphoneCore *lc;
 	char  *peer;
 	LinphoneAddress *peer_url;
-	SalOp *op;
 	void * user_data;
+	MSList *messages_hist;
 };
 
+
+	
 struct _LinphoneFriend{
 	LinphoneAddress *uri;
 	SalOp *insub;
@@ -305,28 +434,36 @@ typedef struct sip_config
 	MSList *proxies;
 	MSList *deleted_proxies;
 	int inc_timeout;	/*timeout after an un-answered incoming call is rejected*/
+	int in_call_timeout;	/*timeout after a call is hangup */
+	int delayed_timeout; 	/*timeout after a delayed call is resumed */
 	unsigned int keepalive_period; /* interval in ms between keep alive messages sent to the proxy server*/
 	LCSipTransports transports;
-	bool_t use_info;
-	bool_t use_rfc2833;	/*force RFC2833 to be sent*/
 	bool_t guess_hostname;
 	bool_t loopback_only;
 	bool_t ipv6_enabled;
 	bool_t sdp_200_ack;
 	bool_t register_only_when_network_is_up;
+	bool_t register_only_when_upnp_is_ok;
 	bool_t ping_with_options;
 	bool_t auto_net_state_mon;
+	bool_t tcp_tls_keepalive;
 } sip_config_t;
 
 typedef struct rtp_config
 {
-	int audio_rtp_port;
-	int video_rtp_port;
+	int audio_rtp_min_port;
+	int audio_rtp_max_port;
+	int video_rtp_min_port;
+	int video_rtp_max_port;
 	int audio_jitt_comp;  /*jitter compensation*/
 	int video_jitt_comp;  /*jitter compensation*/
 	int nortp_timeout;
+	int disable_upnp;
 	bool_t rtp_no_xmit_on_audio_mute;
                               /* stop rtp xmit when audio muted */
+	bool_t audio_adaptive_jitt_comp_enabled;
+	bool_t video_adaptive_jitt_comp_enabled;
+	bool_t pad;
 }rtp_config_t;
 
 
@@ -341,7 +478,6 @@ typedef struct net_config
 	int upload_bw;
 	int firewall_policy;
 	int mtu;
-	int down_ptime;
 	bool_t nat_sdp_only;
 }net_config_t;
 
@@ -354,10 +490,11 @@ typedef struct sound_config
 	struct _MSSndCard * lsd_card; /* dummy playback card for Linphone Sound Daemon extension */
 	const char **cards;
 	int latency;	/* latency in samples of the current used sound device */
+	float soft_play_lev; /*playback gain in db.*/
+	float soft_mic_lev; /*mic gain in db.*/
 	char rec_lev;
 	char play_lev;
 	char ring_lev;
-	char soft_play_lev;
 	char source;
 	char *local_ring;
 	char *remote_ring;
@@ -407,6 +544,7 @@ struct _LinphoneConference{
 	MSAudioConference *conf;
 	AudioStream *local_participant;
 	MSAudioEndpoint *local_endpoint;
+	MSAudioEndpoint *record_endpoint;
 	RtpProfile *local_dummy_profile;
 	bool_t local_muted;
 };
@@ -419,6 +557,7 @@ struct _LinphoneCore
 	Sal *sal;
 	LinphoneGlobalState state;
 	struct _LpConfig *config;
+	RtpProfile *default_profile;
 	net_config_t net_conf;
 	sip_config_t sip_conf;
 	rtp_config_t rtp_conf;
@@ -464,25 +603,41 @@ struct _LinphoneCore
 	MSList *hooks;
 	LinphoneConference conf_ctx;
 	char* zrtp_secrets_cache;
+	LinphoneVideoPolicy video_policy;
 	bool_t use_files;
 	bool_t apply_nat_settings;
 	bool_t initial_subscribes_sent;
 	bool_t bl_refresh;
+	
 	bool_t preview_finished;
 	bool_t auto_net_state_mon;
 	bool_t network_reachable;
 	bool_t use_preview_window;
-	int device_rotation;
+	
+	time_t network_last_check;
+	bool_t network_last_status;
+
 	bool_t ringstream_autorelease;
+	bool_t pad[3];
+	int device_rotation;
 	int max_calls;
 	LinphoneTunnel *tunnel;
+	char* device_id;
+	MSList *last_recv_msg_ids;
+	char *chat_db_file;
+#ifdef MSG_STORAGE_ENABLED
+	sqlite3 *db;
+#endif
+#ifdef BUILD_UPNP
+	UpnpContext *upnp;
+#endif //BUILD_UPNP
 };
 
 LinphoneTunnel *linphone_core_tunnel_new(LinphoneCore *lc);
 void linphone_tunnel_destroy(LinphoneTunnel *tunnel);
 void linphone_tunnel_configure(LinphoneTunnel *tunnel);
 void linphone_tunnel_enable_logs_with_handler(LinphoneTunnel *tunnel, bool_t enabled, OrtpLogFunc logHandler);
-	
+
 bool_t linphone_core_can_we_add_call(LinphoneCore *lc);
 int linphone_core_add_call( LinphoneCore *lc, LinphoneCall *call);
 int linphone_core_del_call( LinphoneCore *lc, LinphoneCall *call);
@@ -490,10 +645,7 @@ int linphone_core_set_as_current_call(LinphoneCore *lc, LinphoneCall *call);
 int linphone_core_get_calls_nb(const LinphoneCore *lc);
 
 void linphone_core_set_state(LinphoneCore *lc, LinphoneGlobalState gstate, const char *message);
-
-SalMediaDescription *create_local_media_description(LinphoneCore *lc, LinphoneCall *call);
-void update_local_media_description(LinphoneCore *lc, LinphoneCall *call, SalMediaDescription **md);
-
+void linphone_call_make_local_media_description(LinphoneCore *lc, LinphoneCall *call);
 void linphone_core_update_streams(LinphoneCore *lc, LinphoneCall *call, SalMediaDescription *new_md);
 
 bool_t linphone_core_is_payload_type_usable_for_bandwidth(LinphoneCore *lc, PayloadType *pt,  int bandwidth_limit);
@@ -505,16 +657,18 @@ struct _EcCalibrator{
 	ms_thread_t thread;
 	MSSndCard *play_card,*capt_card;
 	MSFilter *sndread,*det,*rec;
-	MSFilter *play, *gen, *sndwrite,*resampler;
+	MSFilter *play, *gen, *sndwrite;
+	MSFilter *read_resampler,*write_resampler;
 	MSTicker *ticker;
 	LinphoneEcCalibrationCallback cb;
 	void *cb_data;
-	int recv_count;
-	int sent_count;
+	LinphoneEcCalibrationAudioInit audio_init_cb;
+	LinphoneEcCalibrationAudioUninit audio_uninit_cb;
 	int64_t acc;
 	int delay;
 	unsigned int rate;
 	LinphoneEcCalibratorStatus status;
+	bool_t freq1,freq2,freq3;
 };
 
 typedef struct _EcCalibrator EcCalibrator;
@@ -525,6 +679,8 @@ void ec_calibrator_destroy(EcCalibrator *ecc);
 
 void linphone_call_background_tasks(LinphoneCall *call, bool_t one_second_elapsed);
 void linphone_core_preempt_sound_resources(LinphoneCore *lc);
+int _linphone_core_pause_call(LinphoneCore *lc, LinphoneCall *call);
+
 /*conferencing subsystem*/
 void _post_configure_audio_stream(AudioStream *st, LinphoneCore *lc, bool_t muted);
 /* When a conference participant pause the conference he may send a music.
@@ -535,8 +691,11 @@ void linphone_call_add_to_conf(LinphoneCall *call, bool_t muted);
 void linphone_call_remove_from_conf(LinphoneCall *call);
 void linphone_core_conference_check_uninit(LinphoneCore *lc);
 bool_t linphone_core_sound_resources_available(LinphoneCore *lc);
+void linphone_core_notify_refer_state(LinphoneCore *lc, LinphoneCall *referer, LinphoneCall *newcall);
+unsigned int linphone_core_get_audio_features(LinphoneCore *lc);
 
 void __linphone_core_invalidate_registers(LinphoneCore* lc);
+void _linphone_core_codec_config_write(LinphoneCore *lc);
 
 #define HOLD_OFF	(0)
 #define HOLD_ON		(1)
@@ -546,7 +705,30 @@ void __linphone_core_invalidate_registers(LinphoneCore* lc);
 #endif
 void call_logs_write_to_config_file(LinphoneCore *lc);
 
+int linphone_core_get_edge_bw(LinphoneCore *lc);
+int linphone_core_get_edge_ptime(LinphoneCore *lc);
+void _linphone_call_params_copy(LinphoneCallParams *params, const LinphoneCallParams *refparams);
+void linphone_call_params_uninit(LinphoneCallParams *params);
 
+int linphone_upnp_init(LinphoneCore *lc);
+void linphone_upnp_destroy(LinphoneCore *lc);
+
+#ifdef MSG_STORAGE_ENABLED
+sqlite3 * linphone_message_storage_init();
+#endif
+void linphone_chat_message_store(LinphoneChatMessage *msg);
+void linphone_chat_message_store_state(LinphoneChatMessage *msg);
+void linphone_core_message_storage_init(LinphoneCore *lc);
+void linphone_core_message_storage_close(LinphoneCore *lc);
+
+typedef enum _LinphoneToneID{
+	LinphoneToneBusy,
+	LinphoneToneCallWaiting,
+	LinphoneToneCallOnHold,
+	LinphoneToneCallFailed
+}LinphoneToneID;
+void linphone_core_play_named_tone(LinphoneCore *lc, LinphoneToneID id);
+bool_t linphone_core_tone_indications_enabled(LinphoneCore*lc);
 
 #ifdef __cplusplus
 }
