@@ -30,6 +30,7 @@ int sdp_to_media_description(sdp_message_t *sdp, SalMediaDescription *desc);
 
 struct Sal{
 	SalCallbacks callbacks;
+	SalTransport transport;
 	MSList *calls; /*MSList of SalOp */
 	MSList *registers;/*MSList of SalOp */
 	MSList *out_subscribes;/*MSList of SalOp */
@@ -41,12 +42,17 @@ struct Sal{
 	int keepalive_period;
 	void *up; /*user pointer*/
 	char* rootCa; /* File _or_ folder containing root CA */
+	int dscp;
 	bool_t one_matching_codec;
 	bool_t double_reg;
 	bool_t use_rports;
 	bool_t use_101;
 	bool_t reuse_authorization;
 	bool_t verify_server_certs;
+	bool_t verify_server_cn;
+	bool_t expire_old_contact;
+	bool_t add_dates;
+	bool_t tcp_tls_keepalive;
 };
 
 struct SalOp{
@@ -65,13 +71,14 @@ struct SalOp{
 	 			to retrieve the operation when receiving a response*/
 	char *replaces;
 	char *referred_by;
+	const SalAuthInfo *auth_info;
+	const char *sipfrag_pending;
 	bool_t supports_session_timers;
 	bool_t sdp_offering;
 	bool_t reinvite;
 	bool_t masquerade_via;
 	bool_t auto_answer_asked;
 	bool_t terminated;
-	const SalAuthInfo *auth_info;
 };
 
 void sal_remove_out_subscribe(Sal *sal, SalOp *op);
@@ -87,7 +94,11 @@ void sal_exosip_in_subscription_closed(Sal *sal, eXosip_event_t *ev);
 SalOp * sal_find_out_subscribe(Sal *sal, int sid);
 SalOp * sal_find_in_subscribe(Sal *sal, int nid);
 void sal_exosip_fix_route(SalOp *op);
+void sal_exosip_add_custom_headers(osip_message_t *msg, SalCustomHeader *ch);
+SalCustomHeader * sal_exosip_get_custom_headers(osip_message_t *msg);
 
 void _osip_list_set_empty(osip_list_t *l, void (*freefunc)(void*));
 
+void sal_message_add_route(osip_message_t *msg, const char *proxy);
+
 #endif
