@@ -17,11 +17,8 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-#ifdef HAVE_CONFIG_H
-#include "../config.h"
-#endif
-
 #include "linphonecore.h"
+#include "private.h"
 #include <ctype.h>
 
 static void sip_login_init_instance(SipSetupContext *ctx){
@@ -30,13 +27,13 @@ static void sip_login_init_instance(SipSetupContext *ctx){
 	linphone_proxy_config_enable_register(cfg,FALSE);
 }
 
-static void guess_display_name(osip_from_t *from){
-	char *dn=(char*)osip_malloc(strlen(from->url->username)+3);
-	char *it=from->url->username;
+static void guess_display_name(LinphoneAddress *from){
+	char *dn=(char*)ms_malloc(strlen(linphone_address_get_username(from))+3);
+	const char *it;
 	char *wptr=dn;
 	bool_t begin=TRUE;
 	bool_t surname=0;
-	for(it=from->url->username;*it!='\0';++it){
+	for(it=linphone_address_get_username(from);*it!='\0';++it){
 		if (begin){
 			*wptr=toupper(*it);
 			begin=FALSE;
@@ -48,32 +45,34 @@ static void guess_display_name(osip_from_t *from){
 		}else *wptr=*it;
 		wptr++;
 	}
-	if (from->displayname!=NULL) osip_free(from->displayname);
-	from->displayname=dn;
+	linphone_address_set_display_name(from,dn);
+	ms_free(dn);
 }
 
 static int sip_login_do_login(SipSetupContext * ctx, const char *uri, const char *passwd){
 	LinphoneProxyConfig *cfg=sip_setup_context_get_proxy_config(ctx);
 	LinphoneCore *lc=linphone_proxy_config_get_core(cfg);
 	LinphoneAuthInfo *auth;
-	osip_from_t *parsed_uri;
+	LinphoneAddress *parsed_uri;
 	char *tmp;
 
-	osip_from_init(&parsed_uri);
-	osip_from_parse(parsed_uri,uri);
-	if (parsed_uri->displayname==NULL || strlen(parsed_uri->displayname)==0){
+	parsed_uri=linphone_address_new(uri);
+	if (parsed_uri==NULL){
+		return -1;
+	}
+	if (linphone_address_get_display_name(parsed_uri)!=NULL){
 		guess_display_name(parsed_uri);
 	}
-	osip_from_to_str(parsed_uri,&tmp);
+	tmp=linphone_address_as_string(parsed_uri);
 	linphone_proxy_config_set_identity(cfg,tmp);
 	if (passwd ) {
-		auth=linphone_auth_info_new(parsed_uri->url->username,NULL,passwd,NULL,NULL);
+		auth=linphone_auth_info_new(linphone_address_get_username(parsed_uri),NULL,passwd,NULL,NULL);
 		linphone_core_add_auth_info(lc,auth);
 	}
 	linphone_proxy_config_enable_register(cfg,TRUE);
 	linphone_proxy_config_done(cfg);
-	osip_free(tmp);
-	osip_from_free(parsed_uri);
+	ms_free(tmp);
+	linphone_address_destroy(parsed_uri);
 	ms_message("SipLogin: done");
 	return 0;
 }
@@ -86,6 +85,9 @@ static int sip_login_do_logout(SipSetupContext * ctx){
 }
 
 /* a simple SipSetup built-in plugin to allow specify the user/password for proxy config at runtime*/
+
+#ifndef _MSC_VER
+
 SipSetup linphone_sip_login={
 	.name="SipLogin",
 	.capabilities=SIP_SETUP_CAP_LOGIN,
@@ -94,3 +96,29 @@ SipSetup linphone_sip_login={
 	.logout_account=sip_login_do_logout
 };
 
+#else
+SipSetup linphone_sip_login={
+	"SipLogin",
+	SIP_SETUP_CAP_LOGIN,
+	0,
+	NULL,
+	NULL,
+	sip_login_init_instance,
+	NULL,
+	NULL,
+	NULL,
+	sip_login_do_login,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	sip_login_do_logout
+};
+
+
+
+#endif
