@@ -19,7 +19,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 /*
 *  C Implementation: incall_frame
 *
-* Description: 
+* Description:
 *
 *
 * Author: Simon Morlat <simon.morlat@linphone.org>, (C) 2009
@@ -50,7 +50,8 @@ LinphoneCall *linphone_gtk_get_currently_displayed_call(gboolean *is_conf){
 		if (page!=NULL){
 			LinphoneCall *call=(LinphoneCall*)g_object_get_data(G_OBJECT(page),"call");
 			if (call==NULL){
-				if (page==g_object_get_data(G_OBJECT(main_window),"conference_tab")){
+				GtkWidget *conf_frame=(GtkWidget *)g_object_get_data(G_OBJECT(main_window),"conf_frame");
+				if (conf_frame==page){
 					if (is_conf)
 						*is_conf=TRUE;
 					return NULL;
@@ -64,7 +65,7 @@ LinphoneCall *linphone_gtk_get_currently_displayed_call(gboolean *is_conf){
 
 static GtkWidget *make_tab_header(int number){
 	GtkWidget *w=gtk_hbox_new (FALSE,0);
-	GtkWidget *i=create_pixmap ("status-green.png");
+	GtkWidget *i=create_pixmap ("startcall-small.png");
 	GtkWidget *l;
 	gchar *text=g_strdup_printf(_("Call #%i"),number);
 	l=gtk_label_new (text);
@@ -74,18 +75,43 @@ static GtkWidget *make_tab_header(int number){
 	return w;
 }
 
+void linphone_gtk_call_update_tab_header(LinphoneCall *call,gboolean pause){
+	GtkWidget *w=(GtkWidget*)linphone_call_get_user_pointer(call);
+	GtkWidget *main_window=linphone_gtk_get_main_window();
+	GtkNotebook *notebook=GTK_NOTEBOOK(linphone_gtk_get_widget(main_window,"viewswitch"));
+	gint call_index=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w),"call_index"));
+	GtkWidget *new_label=gtk_hbox_new (FALSE,0);
+	GtkWidget *i=NULL;
+	GtkWidget *l;
+	gchar *text;
+	
+	if(pause){
+		i=gtk_image_new_from_stock(GTK_STOCK_MEDIA_PAUSE,GTK_ICON_SIZE_SMALL_TOOLBAR);
+	} else {
+		i=create_pixmap ("startcall-small.png");
+	}
+	
+	text=g_strdup_printf(_("Call #%i"),call_index);
+	l=gtk_label_new (text);
+	gtk_box_pack_start (GTK_BOX(new_label),i,FALSE,FALSE,0);
+	gtk_box_pack_end(GTK_BOX(new_label),l,TRUE,TRUE,0);
+
+	gtk_notebook_set_tab_label(notebook,w,new_label);
+	gtk_widget_show_all(new_label);
+}
+
 static void linphone_gtk_in_call_set_animation_image(GtkWidget *callview, const char *image_name, gboolean is_stock){
 	GtkWidget *container=linphone_gtk_get_widget(callview,"in_call_animation");
 	GList *elem=gtk_container_get_children(GTK_CONTAINER(container));
 	GtkWidget *image;
-	
+
 	if (!is_stock){
 		if (image_name==NULL){
 			gtk_widget_hide(container);
 		}
 		image=create_pixmap(image_name);
 	}else
-		image=gtk_image_new_from_stock(image_name,GTK_ICON_SIZE_DIALOG);
+		image=gtk_image_new_from_stock(image_name,GTK_ICON_SIZE_DND);
 	if (elem)
 		gtk_widget_destroy((GtkWidget*)elem->data);
 	gtk_widget_show(image);
@@ -107,19 +133,18 @@ static void linphone_gtk_in_call_set_animation_spinner(GtkWidget *callview){
 #endif
 }
 
-
 static void linphone_gtk_transfer_call(LinphoneCall *dest_call){
 	LinphoneCall *call=linphone_gtk_get_currently_displayed_call(NULL);
 	if (call) linphone_core_transfer_call_to_another(linphone_gtk_get_core(),call,dest_call);
 }
 
-static void transfer_button_clicked(GtkWidget *button, gpointer call_ref){
+void transfer_button_clicked(GtkWidget *button, gpointer call_ref){
 	GtkWidget *menu_item;
 	GtkWidget *menu=gtk_menu_new();
 	LinphoneCall *call=(LinphoneCall*)call_ref;
 	LinphoneCore *lc=linphone_gtk_get_core();
 	const MSList *elem=linphone_core_get_calls(lc);
-	
+
 	for(;elem!=NULL;elem=elem->next){
 		LinphoneCall *other_call=(LinphoneCall*)elem->data;
 		GtkWidget *call_view=(GtkWidget*)linphone_call_get_user_pointer(other_call);
@@ -136,39 +161,27 @@ static void transfer_button_clicked(GtkWidget *button, gpointer call_ref){
 			g_signal_connect_swapped(G_OBJECT(menu_item),"activate",(GCallback)linphone_gtk_transfer_call,other_call);
 		}
 	}
-	gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,0,
-		gtk_get_current_event_time());
+	gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,0,gtk_get_current_event_time());
 	gtk_widget_show(menu);
 }
 
-
-
 void linphone_gtk_enable_transfer_button(LinphoneCore *lc, gboolean value){
 	const MSList *elem=linphone_core_get_calls(lc);
 	for(;elem!=NULL;elem=elem->next){
 		LinphoneCall *call=(LinphoneCall*)elem->data;
 		GtkWidget *call_view=(GtkWidget*)linphone_call_get_user_pointer(call);
-		GtkWidget *box=linphone_gtk_get_widget (call_view,"mute_pause_buttons");
-		GtkWidget *button=(GtkWidget*)g_object_get_data(G_OBJECT(box),"transfer");
-		if (button && value==FALSE){
-			gtk_widget_destroy(button);
-			button=NULL;
-		}else if (!button && value==TRUE){
-			button=gtk_button_new_with_label (_("Transfer"));
-			//gtk_button_set_image_position(GTK_BUTTON(button),GTK_POS_BOTTOM);
-			gtk_button_set_image(GTK_BUTTON(button),gtk_image_new_from_stock (GTK_STOCK_GO_FORWARD,GTK_ICON_SIZE_BUTTON));
-			g_signal_connect(G_OBJECT(button),"clicked",(GCallback)transfer_button_clicked,call);
-			gtk_widget_show_all(button);
-			gtk_container_add(GTK_CONTAINER(box),button);
+		GtkWidget *button=linphone_gtk_get_widget (call_view,"transfer_button");
+		if(button != NULL){
+			gtk_widget_set_sensitive(button,value);
 		}
-		g_object_set_data(G_OBJECT(box),"transfer",button);
 	}
 }
 
 static void conference_button_clicked(GtkWidget *button, gpointer call_ref){
-	linphone_core_add_all_to_conference(linphone_gtk_get_core());
-	//linphone_core_add_to_conference(linphone_gtk_get_core(),(LinphoneCall*)call_ref);
 	gtk_widget_set_sensitive(button,FALSE);
+	g_object_set_data(G_OBJECT(linphone_gtk_get_main_window()),"conf_frame",NULL);
+	linphone_core_add_all_to_conference(linphone_gtk_get_core());
+	
 }
 
 void linphone_gtk_enable_conference_button(LinphoneCore *lc, gboolean value){
@@ -176,23 +189,159 @@ void linphone_gtk_enable_conference_button(LinphoneCore *lc, gboolean value){
 	for(;elem!=NULL;elem=elem->next){
 		LinphoneCall *call=(LinphoneCall*)elem->data;
 		GtkWidget *call_view=(GtkWidget*)linphone_call_get_user_pointer(call);
-		GtkWidget *box=linphone_gtk_get_widget (call_view,"mute_pause_buttons");
-		GtkWidget *button=(GtkWidget*)g_object_get_data(G_OBJECT(box),"conference");
-		if (button && value==FALSE){
-			gtk_widget_destroy(button);
-			button=NULL;
-		}else if (!button && value==TRUE){
-			button=gtk_button_new_with_label (_("Conference"));
-			//gtk_button_set_image_position(GTK_BUTTON(button),GTK_POS_BOTTOM);
-			gtk_button_set_image(GTK_BUTTON(button),gtk_image_new_from_stock (GTK_STOCK_ADD,GTK_ICON_SIZE_BUTTON));
-			g_signal_connect(G_OBJECT(button),"clicked",(GCallback)conference_button_clicked,call);
-			gtk_widget_show_all(button);
-			gtk_container_add(GTK_CONTAINER(box),button);
+		GtkWidget *button=linphone_gtk_get_widget (call_view,"conference_button");
+		if (button != NULL){
+			gtk_widget_set_sensitive(button,value);
 		}
-		g_object_set_data(G_OBJECT(box),"conference",button);
 	}
 }
 
+static void show_used_codecs(GtkWidget *callstats, LinphoneCall *call){
+	const LinphoneCallParams *params=linphone_call_get_current_params(call);
+	if (params){
+		const PayloadType *acodec=linphone_call_params_get_used_audio_codec(params);
+		const PayloadType *vcodec=linphone_call_params_get_used_video_codec(params);
+		GtkWidget *acodec_ui=linphone_gtk_get_widget(callstats,"audio_codec");
+		GtkWidget *vcodec_ui=linphone_gtk_get_widget(callstats,"video_codec");
+		if (acodec){
+			char tmp[64]={0};
+			snprintf(tmp,sizeof(tmp)-1,"%s/%i/%i",acodec->mime_type,acodec->clock_rate,acodec->channels);
+			gtk_label_set_label(GTK_LABEL(acodec_ui),tmp);
+		}else gtk_label_set_label(GTK_LABEL(acodec_ui),_("Not used"));
+		if (vcodec){
+			gtk_label_set_label(GTK_LABEL(vcodec_ui),vcodec->mime_type);
+		}else gtk_label_set_label(GTK_LABEL(vcodec_ui),_("Not used"));
+	}
+}
+
+static const char *ice_state_to_string(LinphoneIceState ice_state){
+	switch(ice_state){
+		case LinphoneIceStateNotActivated:
+			return _("ICE not activated");
+		case LinphoneIceStateFailed:
+			return _("ICE failed");
+		case LinphoneIceStateInProgress:
+			return _("ICE in progress");
+		case LinphoneIceStateReflexiveConnection:
+			return _("Going through one or more NATs");
+		case LinphoneIceStateHostConnection:
+			return _("Direct");
+		case LinphoneIceStateRelayConnection:
+			return _("Through a relay server");
+	}
+	return "invalid";
+}
+
+static const char *upnp_state_to_string(LinphoneUpnpState ice_state){
+	switch(ice_state){
+		case LinphoneUpnpStateIdle:
+			return _("uPnP not activated");
+		case LinphoneUpnpStatePending:
+			return _("uPnP in progress");
+		case LinphoneUpnpStateNotAvailable:
+			return _("uPnp not available");
+		case LinphoneUpnpStateOk:
+			return _("uPnP is running");
+		case LinphoneUpnpStateKo:
+			return _("uPnP failed");
+		default:
+			break;
+	}
+	return "invalid";
+}
+
+static void _refresh_call_stats(GtkWidget *callstats, LinphoneCall *call){
+	const LinphoneCallStats *as=linphone_call_get_audio_stats(call);
+	const LinphoneCallStats *vs=linphone_call_get_video_stats(call);
+	const char *audio_media_connectivity = _("Direct or through server");
+	const char *video_media_connectivity = _("Direct or through server");
+	gboolean has_video=linphone_call_params_video_enabled(linphone_call_get_current_params(call));
+	gchar *tmp=g_strdup_printf(_("download: %f\nupload: %f (kbit/s)"),
+		as->download_bandwidth,as->upload_bandwidth);
+	
+	gtk_label_set_markup(GTK_LABEL(linphone_gtk_get_widget(callstats,"audio_bandwidth_usage")),tmp);
+	g_free(tmp);
+	if (has_video)
+		tmp=g_strdup_printf(_("download: %f\nupload: %f (kbit/s)"),vs->download_bandwidth,vs->upload_bandwidth);
+	else tmp=NULL;
+	gtk_label_set_markup(GTK_LABEL(linphone_gtk_get_widget(callstats,"video_bandwidth_usage")),tmp);
+	if (tmp) g_free(tmp);
+	if(as->upnp_state != LinphoneUpnpStateNotAvailable && as->upnp_state != LinphoneUpnpStateIdle) {
+		audio_media_connectivity = upnp_state_to_string(as->upnp_state);
+	} else if(as->ice_state != LinphoneIceStateNotActivated) {
+		audio_media_connectivity = ice_state_to_string(as->ice_state);
+	}
+	gtk_label_set_text(GTK_LABEL(linphone_gtk_get_widget(callstats,"audio_media_connectivity")),audio_media_connectivity);
+	
+	if (has_video){
+		if(vs->upnp_state != LinphoneUpnpStateNotAvailable && vs->upnp_state != LinphoneUpnpStateIdle) {
+				video_media_connectivity = upnp_state_to_string(vs->upnp_state);
+		} else if(vs->ice_state != LinphoneIceStateNotActivated) {
+			video_media_connectivity = ice_state_to_string(vs->ice_state);
+		}
+	}else video_media_connectivity=NULL;
+	gtk_label_set_text(GTK_LABEL(linphone_gtk_get_widget(callstats,"video_media_connectivity")),video_media_connectivity);
+	
+	if (as->round_trip_delay>0){
+		tmp=g_strdup_printf(_("%.3f seconds"),as->round_trip_delay);
+		gtk_label_set_text(GTK_LABEL(linphone_gtk_get_widget(callstats,"round_trip_time")),tmp);
+		g_free(tmp);
+	}
+}
+
+static gboolean refresh_call_stats(GtkWidget *callstats){
+	LinphoneCall *call=(LinphoneCall*)g_object_get_data(G_OBJECT(callstats),"call");
+	switch (linphone_call_get_state(call)){
+		case LinphoneCallError:
+		case LinphoneCallEnd:
+		case LinphoneCallReleased:
+			gtk_widget_destroy(callstats);
+			return FALSE;
+		break;
+		case LinphoneCallStreamsRunning:
+			_refresh_call_stats(callstats,call);
+		break;
+		default:
+		break;
+	}
+	return TRUE;
+}
+
+static void on_call_stats_destroyed(GtkWidget *call_view){
+	GtkWidget *call_stats=(GtkWidget*)g_object_get_data(G_OBJECT(call_view),"call_stats");
+	LinphoneCall *call=(LinphoneCall*)g_object_get_data(G_OBJECT(call_stats),"call");
+	g_source_remove(GPOINTER_TO_INT(g_object_get_data(G_OBJECT(call_stats),"tid")));
+	g_object_set_data(G_OBJECT(call_view),"call_stats",NULL);
+	linphone_call_unref(call);
+}
+
+static void linphone_gtk_show_call_stats(LinphoneCall *call){
+	GtkWidget *w=(GtkWidget*)linphone_call_get_user_pointer(call);
+	GtkWidget *call_stats=(GtkWidget*)g_object_get_data(G_OBJECT(w),"call_stats");
+	if (call_stats==NULL){
+		guint tid;
+		call_stats=linphone_gtk_create_window("call_statistics");
+		g_object_set_data(G_OBJECT(w),"call_stats",call_stats);
+		g_object_set_data(G_OBJECT(call_stats),"call",linphone_call_ref(call));
+		tid=g_timeout_add(1000,(GSourceFunc)refresh_call_stats,call_stats);
+		g_object_set_data(G_OBJECT(call_stats),"tid",GINT_TO_POINTER(tid));
+		g_signal_connect_swapped(G_OBJECT(call_stats),"destroy",(GCallback)on_call_stats_destroyed,(gpointer)w);
+		show_used_codecs(call_stats,call);
+		refresh_call_stats(call_stats);
+		gtk_widget_show(call_stats);
+	}
+
+}
+
+void linphone_gtk_enable_video_button(LinphoneCall *call, gboolean sensitive, gboolean holdon){
+	GtkWidget *callview=(GtkWidget*)linphone_call_get_user_pointer (call);
+	GtkWidget *button;
+	g_return_if_fail(callview!=NULL);
+	button=linphone_gtk_get_widget(callview,"video_button");
+	gtk_widget_set_sensitive(GTK_WIDGET(button),sensitive);
+	gtk_widget_set_visible(GTK_WIDGET(button),sensitive);
+}
+
 void linphone_gtk_create_in_call_view(LinphoneCall *call){
 	GtkWidget *call_view=linphone_gtk_create_widget("main","in_call_frame");
 	GtkWidget *main_window=linphone_gtk_get_main_window ();
@@ -215,8 +364,63 @@ void linphone_gtk_create_in_call_view(LinphoneCall *call){
 	gtk_notebook_set_current_page(notebook, idx);
 	call_index++;
 	linphone_gtk_enable_hold_button (call,FALSE,TRUE);
+	linphone_gtk_enable_video_button (call,FALSE,TRUE);
 	linphone_gtk_enable_mute_button(
 					GTK_BUTTON(linphone_gtk_get_widget(call_view,"incall_mute")),FALSE);
+
+	GtkWidget *transfer = linphone_gtk_get_widget(call_view,"transfer_button");
+	gtk_button_set_image(GTK_BUTTON(transfer),gtk_image_new_from_stock
+							 (GTK_STOCK_GO_FORWARD,GTK_ICON_SIZE_BUTTON));
+	g_signal_connect(G_OBJECT(transfer),"clicked",(GCallback)transfer_button_clicked,call);
+	gtk_widget_hide(transfer);
+
+	GtkWidget *conf = linphone_gtk_get_widget(call_view,"conference_button");
+	gtk_button_set_image(GTK_BUTTON(conf),gtk_image_new_from_stock (GTK_STOCK_ADD,GTK_ICON_SIZE_BUTTON));
+	g_signal_connect(G_OBJECT(conf),"clicked",(GCallback)conference_button_clicked,call);
+	gtk_widget_hide(conf);
+
+	GtkWidget *button=linphone_gtk_get_widget(call_view,"terminate_call");
+	GtkWidget *image=create_pixmap("stopcall-small.png");
+	gtk_button_set_label(GTK_BUTTON(button),_("Hang up"));
+	gtk_button_set_image(GTK_BUTTON(button),image);
+	gtk_widget_show(image);
+	g_signal_connect_swapped(G_OBJECT(linphone_gtk_get_widget(call_view,"quality_indicator")),"button-press-event",(GCallback)linphone_gtk_show_call_stats,call);
+}
+
+static void video_button_clicked(GtkWidget *button, LinphoneCall *call){
+	gboolean adding=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(button),"adding_video"));
+	LinphoneCore *lc=linphone_call_get_core(call);
+	LinphoneCallParams *params=linphone_call_params_copy(linphone_call_get_current_params(call));
+	gtk_widget_set_sensitive(button,FALSE);
+	linphone_call_params_enable_video(params,adding);
+	linphone_core_update_call(lc,call,params);
+	linphone_call_params_destroy(params);
+}
+
+void linphone_gtk_update_video_button(LinphoneCall *call){
+	GtkWidget *call_view=(GtkWidget*)linphone_call_get_user_pointer(call);
+	GtkWidget *button;
+	const LinphoneCallParams *params=linphone_call_get_current_params(call);
+	gboolean has_video=linphone_call_params_video_enabled(params);
+	if (call_view==NULL) return;
+	button=linphone_gtk_get_widget(call_view,"video_button");
+
+	gtk_button_set_image(GTK_BUTTON(button),
+	gtk_image_new_from_stock(has_video ? GTK_STOCK_REMOVE : GTK_STOCK_ADD,GTK_ICON_SIZE_BUTTON));
+	g_object_set_data(G_OBJECT(button),"adding_video",GINT_TO_POINTER(!has_video));
+	if (!linphone_core_video_supported(linphone_call_get_core(call))){
+		gtk_widget_set_sensitive(button,FALSE);
+		return;
+	}
+	if (GPOINTER_TO_INT(g_object_get_data(G_OBJECT(button),"signal_connected"))==0){
+		g_signal_connect(G_OBJECT(button),"clicked",(GCallback)video_button_clicked,call);
+		g_object_set_data(G_OBJECT(button),"signal_connected",GINT_TO_POINTER(1));
+	}
+	GtkWidget *conf_frame=(GtkWidget *)g_object_get_data(G_OBJECT(linphone_gtk_get_main_window()),"conf_frame");
+	gtk_widget_set_sensitive(button,linphone_call_get_state(call)==LinphoneCallStreamsRunning);
+	if(conf_frame!=NULL){
+		gtk_widget_set_sensitive(button,FALSE);
+	}
 }
 
 void linphone_gtk_remove_in_call_view(LinphoneCall *call){
@@ -227,8 +431,6 @@ void linphone_gtk_remove_in_call_view(LinphoneCall *call){
 	int idx;
 	g_return_if_fail(w!=NULL);
 	idx=gtk_notebook_page_num(GTK_NOTEBOOK(nb),w);
-	gtk_notebook_remove_page (GTK_NOTEBOOK(nb),idx);
-	gtk_widget_destroy(w);
 	if (in_conf){
 		linphone_gtk_unset_from_conference(call);
 	}
@@ -239,13 +441,14 @@ void linphone_gtk_remove_in_call_view(LinphoneCall *call){
 		if (linphone_core_is_in_conference(linphone_gtk_get_core())){
 			/*show the conference*/
 			gtk_notebook_set_current_page(GTK_NOTEBOOK(nb),gtk_notebook_page_num(GTK_NOTEBOOK(nb),
-		                            g_object_get_data(G_OBJECT(main_window),"conference_tab")));
-		}else gtk_notebook_set_current_page(GTK_NOTEBOOK(nb), 0);
+		                            g_object_get_data(G_OBJECT(main_window),"conf_frame")));
+		}else gtk_notebook_prev_page(GTK_NOTEBOOK(nb));
 	}else{
 		/*show the active call*/
-		gtk_notebook_set_current_page(GTK_NOTEBOOK(nb),gtk_notebook_page_num(GTK_NOTEBOOK(nb),
-		                                                                     linphone_call_get_user_pointer(call)));
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(nb),gtk_notebook_page_num(GTK_NOTEBOOK(nb),                                                                     linphone_call_get_user_pointer(call)));
 	}
+	gtk_notebook_remove_page (GTK_NOTEBOOK(nb),idx);
+	gtk_widget_destroy(w);
 }
 
 static void display_peer_name_in_label(GtkWidget *label, const LinphoneAddress *from){
@@ -254,9 +457,9 @@ static void display_peer_name_in_label(GtkWidget *label, const LinphoneAddress *
 	char *uri_label;
 	displayname=linphone_address_get_display_name(from);
 	id=linphone_address_as_string_uri_only(from);
-	
+
 	if (displayname!=NULL){
-		uri_label=g_markup_printf_escaped("<span size=\"large\">%s</span>\n<i>%s</i>", 
+		uri_label=g_markup_printf_escaped("<span size=\"large\">%s</span>\n<i>%s</i>",
 			displayname,id);
 	}else
 		uri_label=g_markup_printf_escaped("<span size=\"large\"><i>%s</i></span>\n",id);
@@ -269,10 +472,10 @@ void linphone_gtk_in_call_view_set_calling(LinphoneCall *call){
 	GtkWidget *status=linphone_gtk_get_widget(callview,"in_call_status");
 	GtkWidget *callee=linphone_gtk_get_widget(callview,"in_call_uri");
 	GtkWidget *duration=linphone_gtk_get_widget(callview,"in_call_duration");
-	
+
 	gtk_label_set_markup(GTK_LABEL(status),_("<b>Calling...</b>"));
 	display_peer_name_in_label(callee,linphone_call_get_remote_address (call));
-	
+
 	gtk_label_set_text(GTK_LABEL(duration),_("00::00::00"));
 	linphone_gtk_in_call_set_animation_spinner(callview);
 }
@@ -286,19 +489,19 @@ void linphone_gtk_in_call_view_set_incoming(LinphoneCall *call){
 
 	gtk_label_set_markup(GTK_LABEL(status),_("<b>Incoming call</b>"));
 	gtk_widget_show_all(linphone_gtk_get_widget(callview,"answer_decline_panel"));
-	gtk_widget_hide(linphone_gtk_get_widget(callview,"mute_pause_buttons"));
+	gtk_widget_hide(linphone_gtk_get_widget(callview,"buttons_panel"));
 	display_peer_name_in_label(callee,linphone_call_get_remote_address (call));
 
 	answer_button=linphone_gtk_get_widget(callview,"accept_call");
-	image=create_pixmap (linphone_gtk_get_ui_config("start_call_icon","startcall-green.png"));
+	image=create_pixmap (linphone_gtk_get_ui_config("start_call_icon","startcall-small.png"));
 	gtk_button_set_label(GTK_BUTTON(answer_button),_("Answer"));
 	gtk_button_set_image(GTK_BUTTON(answer_button),image);
 	gtk_widget_show(image);
-	
-	image=create_pixmap (linphone_gtk_get_ui_config("stop_call_icon","stopcall-red.png"));
+
+	image=create_pixmap (linphone_gtk_get_ui_config("stop_call_icon","stopcall-small.png"));
 	gtk_button_set_image(GTK_BUTTON(linphone_gtk_get_widget(callview,"decline_call")),image);
 	gtk_widget_show(image);
-	
+
 	linphone_gtk_in_call_set_animation_image(callview,GTK_STOCK_DIALOG_INFO,TRUE);
 }
 
@@ -353,8 +556,6 @@ static gboolean linphone_gtk_in_call_view_refresh(LinphoneCall *call){
 	return TRUE;
 }
 
-
-
 typedef struct _volume_ctx{
 	GtkWidget *widget;
 	get_volume_t get_volume;
@@ -408,13 +609,14 @@ void linphone_gtk_uninit_audio_meter(GtkWidget *w){
 void linphone_gtk_in_call_view_enable_audio_view(LinphoneCall *call, gboolean val){
 	GtkWidget *callview=(GtkWidget*)linphone_call_get_user_pointer(call);
 	GtkWidget *audio_view=linphone_gtk_get_widget(callview,"incall_audioview");
-	//GtkWidget *mic=linphone_gtk_get_widget(callview,"incall_mic_icon");
+	GtkWidget *mic=linphone_gtk_get_widget(callview,"incall_mic_icon");
 	GtkWidget *spk=linphone_gtk_get_widget(callview,"incall_spk_icon");
 	GtkWidget *mic_level=linphone_gtk_get_widget(callview,"mic_audiolevel");
 	GtkWidget *spk_level=linphone_gtk_get_widget(callview,"spk_audiolevel");
 	GdkPixbuf *pbuf;
-	//gtk_image_set_from_pixbuf(GTK_IMAGE(mic),(pbuf=create_pixbuf("mic_active.png")));
-	//g_object_unref(pbuf);
+
+	gtk_image_set_from_pixbuf(GTK_IMAGE(mic),(pbuf=create_pixbuf("mic_active.png")));
+	g_object_unref(pbuf);
 	if (val){
 		gtk_image_set_from_pixbuf(GTK_IMAGE(spk),(pbuf=create_pixbuf("speaker.png")));
 		g_object_unref(pbuf);
@@ -460,7 +662,7 @@ void linphone_gtk_in_call_view_show_encryption(LinphoneCall *call){
 			gtk_button_set_label(GTK_BUTTON(verify_button),
 			                     verified ? _("Set unverified") : _("Set verified"));
 			gtk_widget_show_all(encryption_box);
-		}	
+		}
 		break;
 		default:
 			gtk_widget_hide_all(encryption_box);
@@ -474,24 +676,41 @@ void linphone_gtk_in_call_view_set_in_call(LinphoneCall *call){
 	GtkWidget *duration=linphone_gtk_get_widget(callview,"in_call_duration");
 	guint taskid=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(callview),"taskid"));
 	gboolean in_conf=linphone_call_params_local_conference_mode(linphone_call_get_current_params(call));
-	
-	display_peer_name_in_label(callee,linphone_call_get_remote_address (call));
+	GtkWidget *call_stats=(GtkWidget*)g_object_get_data(G_OBJECT(callview),"call_stats");
 
-	gtk_widget_set_visible(linphone_gtk_get_widget(callview,"mute_pause_buttons"),!in_conf);
+	display_peer_name_in_label(callee,linphone_call_get_remote_address (call));
+	
 	gtk_widget_hide(linphone_gtk_get_widget(callview,"answer_decline_panel"));
 	gtk_label_set_markup(GTK_LABEL(status),in_conf ? _("In conference") : _("<b>In call</b>"));
+	
+	gtk_widget_set_sensitive(linphone_gtk_get_widget(callview,"conference_button"),!in_conf);
+	gtk_widget_set_sensitive(linphone_gtk_get_widget(callview,"transfer_button"),!in_conf);
 
 	gtk_label_set_text(GTK_LABEL(duration),_("00::00::00"));
 	linphone_gtk_in_call_set_animation_image(callview,GTK_STOCK_MEDIA_PLAY,TRUE);
+	linphone_gtk_call_update_tab_header(call,FALSE);
 	linphone_gtk_enable_mute_button(
-					GTK_BUTTON(linphone_gtk_get_widget(callview,"incall_mute")),!in_conf);
+					GTK_BUTTON(linphone_gtk_get_widget(callview,"incall_mute")),TRUE);
+	
 	if (taskid==0){
 		taskid=g_timeout_add(250,(GSourceFunc)linphone_gtk_in_call_view_refresh,call);
 		g_object_set_data(G_OBJECT(callview),"taskid",GINT_TO_POINTER(taskid));
 	}
 	linphone_gtk_in_call_view_enable_audio_view(call, !in_conf);
 	linphone_gtk_in_call_view_show_encryption(call);
-	if (in_conf) linphone_gtk_set_in_conference(call);
+	if (in_conf){
+		linphone_gtk_set_in_conference(call);
+		gtk_widget_set_sensitive(linphone_gtk_get_widget(callview,"incall_mute"),FALSE);
+		gtk_widget_set_sensitive(linphone_gtk_get_widget(callview,"hold_call"),FALSE);
+	}else{
+		linphone_gtk_unset_from_conference(call); /*in case it was previously*/
+		gtk_widget_set_sensitive(linphone_gtk_get_widget(callview,"incall_mute"),TRUE);
+		gtk_widget_set_sensitive(linphone_gtk_get_widget(callview,"hold_call"),TRUE);
+	}
+	gtk_widget_show_all(linphone_gtk_get_widget(callview,"buttons_panel"));
+	if (!in_conf) gtk_widget_show_all(linphone_gtk_get_widget(callview,"record_hbox"));
+	else gtk_widget_hide(linphone_gtk_get_widget(callview,"record_hbox"));
+	if (call_stats) show_used_codecs(call_stats,call);
 }
 
 void linphone_gtk_in_call_view_set_paused(LinphoneCall *call){
@@ -499,6 +718,7 @@ void linphone_gtk_in_call_view_set_paused(LinphoneCall *call){
 	GtkWidget *status=linphone_gtk_get_widget(callview,"in_call_status");
 	gtk_widget_hide(linphone_gtk_get_widget(callview,"answer_decline_panel"));
 	gtk_label_set_markup(GTK_LABEL(status),_("<b>Paused call</b>"));
+
 	linphone_gtk_in_call_set_animation_image(callview,GTK_STOCK_MEDIA_PAUSE,TRUE);
 }
 
@@ -521,10 +741,12 @@ static gboolean in_call_view_terminated(LinphoneCall *call){
 
 void linphone_gtk_in_call_view_terminate(LinphoneCall *call, const char *error_msg){
 	GtkWidget *callview=(GtkWidget*)linphone_call_get_user_pointer(call);
+	if(callview==NULL) return;
 	GtkWidget *status=linphone_gtk_get_widget(callview,"in_call_status");
 	guint taskid=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(callview),"taskid"));
-	gboolean in_conf=linphone_call_params_local_conference_mode(linphone_call_get_current_params(call));	
+	gboolean in_conf=linphone_call_params_local_conference_mode(linphone_call_get_current_params(call));
 
+	if (status==NULL) return;
 	if (error_msg==NULL)
 		gtk_label_set_markup(GTK_LABEL(status),_("<b>Call ended.</b>"));
 	else{
@@ -534,9 +756,12 @@ void linphone_gtk_in_call_view_terminate(LinphoneCall *call, const char *error_m
 	}
 	linphone_gtk_in_call_set_animation_image(callview,
 	           linphone_gtk_get_ui_config("stop_call_icon","stopcall-red.png"),FALSE);
-	
+
 	gtk_widget_hide(linphone_gtk_get_widget(callview,"answer_decline_panel"));
+	gtk_widget_hide(linphone_gtk_get_widget(callview,"record_hbox"));
+	gtk_widget_hide(linphone_gtk_get_widget(callview,"buttons_panel"));
 	gtk_widget_hide(linphone_gtk_get_widget(callview,"incall_audioview"));
+	gtk_widget_hide(linphone_gtk_get_widget(callview,"quality_indicator"));
 	linphone_gtk_enable_mute_button(
 		GTK_BUTTON(linphone_gtk_get_widget(callview,"incall_mute")),FALSE);
 	linphone_gtk_enable_hold_button(call,FALSE,TRUE);
@@ -547,6 +772,28 @@ void linphone_gtk_in_call_view_terminate(LinphoneCall *call, const char *error_m
 		linphone_gtk_terminate_conference_participant(call);
 }
 
+void linphone_gtk_in_call_view_set_transfer_status(LinphoneCall *call,LinphoneCallState cstate){
+	GtkWidget *callview=(GtkWidget*)linphone_call_get_user_pointer(call);
+	if (callview){
+		GtkWidget *duration=linphone_gtk_get_widget(callview,"in_call_duration");
+		const char *transfer_status="unknown";
+		switch(cstate){
+			case LinphoneCallOutgoingProgress:
+				transfer_status=_("Transfer in progress");
+			break;
+			case LinphoneCallConnected:
+				transfer_status=_("Transfer done.");
+			break;
+			case LinphoneCallError:
+				transfer_status=_("Transfer failed.");
+			break;
+			default:
+			break;
+		}
+		gtk_label_set_text(GTK_LABEL(duration),transfer_status);
+	}
+}
+
 void linphone_gtk_draw_mute_button(GtkButton *button, gboolean active){
 	g_object_set_data(G_OBJECT(button),"active",GINT_TO_POINTER(active));
 	if (active){
@@ -572,8 +819,7 @@ void linphone_gtk_mute_clicked(GtkButton *button){
 	linphone_gtk_draw_mute_button(button,!active);
 }
 
-void linphone_gtk_enable_mute_button(GtkButton *button, gboolean sensitive)
-{
+void linphone_gtk_enable_mute_button(GtkButton *button, gboolean sensitive){
 	/*gtk_widget_set_sensitive(GTK_WIDGET(button),sensitive);*/
 	gtk_widget_set_visible(GTK_WIDGET(button),sensitive);
 	linphone_gtk_draw_mute_button(button,FALSE);
@@ -601,6 +847,7 @@ void linphone_gtk_draw_hold_button(GtkButton *button, gboolean active){
 void linphone_gtk_hold_clicked(GtkButton *button){
 	int active=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(button),"active"));
 	LinphoneCall *call=linphone_gtk_get_currently_displayed_call(NULL);
+	linphone_gtk_call_update_tab_header(call,active);
 	if (!call) return;
 	if(!active)
 	{
@@ -616,8 +863,59 @@ void linphone_gtk_enable_hold_button(LinphoneCall *call, gboolean sensitive, gbo
 	GtkWidget *callview=(GtkWidget*)linphone_call_get_user_pointer (call);
 	GtkWidget *button;
 	g_return_if_fail(callview!=NULL);
+	linphone_gtk_call_update_tab_header(call,!holdon);
 	button=linphone_gtk_get_widget(callview,"hold_call");
 	gtk_widget_set_sensitive(GTK_WIDGET(button),sensitive);
 	gtk_widget_set_visible(GTK_WIDGET(button),sensitive);
 	linphone_gtk_draw_hold_button(GTK_BUTTON(button),!holdon);
 }
+
+void linphone_gtk_call_statistics_closed(GtkWidget *call_stats){
+	gtk_widget_destroy(call_stats);
+}
+
+void linphone_gtk_record_call_toggled(GtkWidget *button){
+	gboolean active=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
+	gboolean is_conf=FALSE;
+	const char *filepath;
+	gchar *message;
+	LinphoneCore *lc=linphone_gtk_get_core();
+	LinphoneCall *call=linphone_gtk_get_currently_displayed_call(&is_conf);
+	GtkWidget *callview;
+	GtkWidget *label;
+	if (call){
+		callview=(GtkWidget*)linphone_call_get_user_pointer (call);
+		const LinphoneCallParams *params=linphone_call_get_current_params(call);
+		filepath=linphone_call_params_get_record_file(params);
+		label=linphone_gtk_get_widget(callview,"record_status");
+	}else if (is_conf){
+		GtkWidget *mw=linphone_gtk_get_main_window();
+		callview=(GtkWidget *)g_object_get_data(G_OBJECT(linphone_gtk_get_main_window()),"conf_frame");
+		label=linphone_gtk_get_widget(callview,"conf_record_status");
+		filepath=(const char*)g_object_get_data(G_OBJECT(mw),"conf_record_path");
+		if (filepath==NULL){
+			filepath=linphone_gtk_get_record_path(NULL,TRUE);
+			g_object_set_data_full(G_OBJECT(mw),"conf_record_path",(char*)filepath,g_free);
+		}
+	}else{
+		g_warning("linphone_gtk_record_call_toggled(): bug.");
+		return;
+	}
+	message=g_strdup_printf(_("<small><i>Recording into\n%s %s</i></small>"),filepath,active ? "" : _("(Paused)"));
+	
+	if (active){
+		if (call)
+			linphone_call_start_recording(call);
+		else
+			linphone_core_start_conference_recording(lc,filepath);
+	}else {
+		if (call)
+			linphone_call_stop_recording(call);
+		else
+			linphone_core_stop_conference_recording(lc);
+		
+	}
+	gtk_label_set_markup(GTK_LABEL(label),message);
+	g_free(message);
+}
+
