@@ -23,6 +23,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <sys/stat.h>
 #include <sys/types.h>
 #endif
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 
 extern gchar *linphone_logfile;
 
@@ -206,6 +207,7 @@ static void linphone_gtk_log_file(OrtpLogLevel lev, const char *msg)
 		 case of a crash (which is one of the main reasons we have a
 		     log facility in the first place). */
 		fprintf(outlog, "[%s] [%s] %s\n", date, lname, msg);
+		fflush(outlog);
 	}
 }
 
