@@ -36,7 +36,7 @@ typedef struct _MSQueue
 }MSQueue;
 
 
-MSQueue * ms_queue_new(struct _MSFilter *f1, int pin1, struct _MSFilter *f2, int pin2 );
+MS2_PUBLIC MSQueue * ms_queue_new(struct _MSFilter *f1, int pin1, struct _MSFilter *f2, int pin2 );
 
 static inline mblk_t *ms_queue_get(MSQueue *q){
 	return getq(&q->q);
@@ -83,6 +83,8 @@ MS2_PUBLIC void ms_queue_destroy(MSQueue *q);
 #define mblk_get_marker_info(m)	      ((m)->reserved2&0x1) /*bit 1*/
 #define mblk_set_precious_flag(m,bit)    __mblk_set_flag(m,1,bit)  /*use to prevent mirroring*/
 #define mblk_get_precious_flag(m)    (((m)->reserved2)>>1 & 0x1) /*bit 2*/
+#define mblk_set_plc_flag(m,bit)    __mblk_set_flag(m,2,bit)  /*use to mark a plc generated block*/
+#define mblk_get_plc_flag(m)    (((m)->reserved2)>>1 & 0x2) /*bit 2*/
 #define mblk_set_cseq(m,value) (m)->reserved2=(m)->reserved2| ((value&0xFFFF)<<16);	
 #define mblk_get_cseq(m) ((m)->reserved2>>16)
 	
@@ -97,7 +99,7 @@ typedef struct _MSBufferizer MSBufferizer;
 MS2_PUBLIC MSBufferizer * ms_bufferizer_new(void);
 
 /*initialize in memory */
-void ms_bufferizer_init(MSBufferizer *obj);
+MS2_PUBLIC void ms_bufferizer_init(MSBufferizer *obj);
 
 MS2_PUBLIC void ms_bufferizer_put(MSBufferizer *obj, mblk_t *m);
 
@@ -116,7 +118,7 @@ MS2_PUBLIC void ms_bufferizer_skip_bytes(MSBufferizer *obj, int bytes);
 /* purge all data pending in the bufferizer */
 MS2_PUBLIC void ms_bufferizer_flush(MSBufferizer *obj);
 
-void ms_bufferizer_uninit(MSBufferizer *obj);
+MS2_PUBLIC void ms_bufferizer_uninit(MSBufferizer *obj);
 
 MS2_PUBLIC void ms_bufferizer_destroy(MSBufferizer *obj);
 
