@@ -22,22 +22,29 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define msrtp_hh
 
 #include <mediastreamer2/msfilter.h>
-#include <mediastreamer2/ice.h>
 #include <ortp/ortp.h>
 
-#define MS_RTP_RECV_SET_SESSION		MS_FILTER_METHOD(MS_RTP_RECV_ID,0,RtpSession*)
+#define MS_RTP_RECV_SET_SESSION			MS_FILTER_METHOD(MS_RTP_RECV_ID,0,RtpSession*)
 
-#define MS_RTP_SEND_SET_SESSION		MS_FILTER_METHOD(MS_RTP_SEND_ID,0,RtpSession*)
+#define MS_RTP_RECV_RESET_JITTER_BUFFER		MS_FILTER_METHOD_NO_ARG(MS_RTP_RECV_ID,1)
 
-#define MS_RTP_SEND_SEND_DTMF		MS_FILTER_METHOD(MS_RTP_SEND_ID,1,const char)
 
-#define MS_RTP_SEND_MUTE_MIC		MS_FILTER_METHOD_NO_ARG(MS_RTP_SEND_ID,3)
 
-#define MS_RTP_SEND_UNMUTE_MIC		MS_FILTER_METHOD_NO_ARG(MS_RTP_SEND_ID,4)
+
+#define MS_RTP_SEND_SET_SESSION			MS_FILTER_METHOD(MS_RTP_SEND_ID,0,RtpSession*)
+
+#define MS_RTP_SEND_SEND_DTMF			MS_FILTER_METHOD(MS_RTP_SEND_ID,1,const char)
+
+#define MS_RTP_SEND_SET_DTMF_DURATION		MS_FILTER_METHOD(MS_RTP_SEND_ID,2,int)
+
+#define MS_RTP_SEND_MUTE_MIC			MS_FILTER_METHOD_NO_ARG(MS_RTP_SEND_ID,3)
+
+#define MS_RTP_SEND_UNMUTE_MIC			MS_FILTER_METHOD_NO_ARG(MS_RTP_SEND_ID,4)
 
 #define MS_RTP_SEND_SET_RELAY_SESSION_ID	MS_FILTER_METHOD(MS_RTP_SEND_ID,5,const char *)
 
-#define MS_RTP_SEND_SET_DTMF_DURATION	MS_FILTER_METHOD(MS_RTP_SEND_ID,1,int)
+
+
 
 extern MSFilterDesc ms_rtp_send_desc;
 extern MSFilterDesc ms_rtp_recv_desc;
