@@ -121,9 +121,10 @@ struct _MSSndCardDesc{
  */
 typedef struct _MSSndCardDesc MSSndCardDesc;
 
-#define MS_SND_CARD_CAP_DISABLED (0)
-#define MS_SND_CARD_CAP_CAPTURE (1)
-#define MS_SND_CARD_CAP_PLAYBACK (1<<1)
+#define MS_SND_CARD_CAP_DISABLED (0) /**<This soundcard is disabled.*/
+#define MS_SND_CARD_CAP_CAPTURE (1) /**<This sound card can capture sound */
+#define MS_SND_CARD_CAP_PLAYBACK (1<<1) /**<This sound card can playback sound */
+#define MS_SND_CARD_CAP_BUILTIN_ECHO_CANCELLER (1<<2) /**<This sound card has built-in echo cancellation*/
 
 struct _MSSndCard{
 	MSSndCardDesc *desc;
@@ -131,6 +132,8 @@ struct _MSSndCard{
 	char *id;
 	unsigned int capabilities;
 	void *data;
+	int preferred_sample_rate;
+	int latency;
 };
 
 /**
@@ -232,6 +235,7 @@ MS2_PUBLIC void ms_snd_card_manager_register_desc(MSSndCardManager *m, MSSndCard
 **/
 MS2_PUBLIC void ms_snd_card_manager_reload(MSSndCardManager *m);
 
+
 /** @} */
 
 /**
@@ -332,6 +336,7 @@ MS2_PUBLIC const char *ms_snd_card_get_string_id(MSSndCard *obj);
  *   MS_SND_CARD_CAP_CAPTURE
  *   MS_SND_CARD_CAP_PLAYBACK
  *   MS_SND_CARD_CAP_CAPTURE|MS_SND_CARD_CAP_PLAYBACK
+ *   MS_SND_CARD_CAP_BUILTIN_ECHO_CANCELLER
  * </PRE>
  *
  * @param obj    A sound card object.
@@ -340,6 +345,15 @@ MS2_PUBLIC const char *ms_snd_card_get_string_id(MSSndCard *obj);
  */
 MS2_PUBLIC unsigned int ms_snd_card_get_capabilities(const MSSndCard *obj);
 
+/**
+ * Returns the sound card minimal latency (playback+record), in milliseconds.
+ * This value is to be used by the software echo cancellers to know where to search for the echo (optimization).
+ * Typically, an echo shall not be found before the value returned by this function.
+ * If this value is not known, then it should return 0.
+ * @param obj    A sound card object.
+**/
+MS2_PUBLIC int ms_snd_card_get_minimal_latency(MSSndCard *obj);
+
 /**
  * Set some mixer level value.
  *
@@ -425,6 +439,26 @@ MS2_PUBLIC int ms_snd_card_set_control(MSSndCard *obj, MSSndCardControlElem e, i
  */
 MS2_PUBLIC int ms_snd_card_get_control(MSSndCard *obj, MSSndCardControlElem e);
 
+/**
+ * Get preferred sample rate
+ *
+ * @param obj      A sound card object.
+ *
+ * Returns: return sample rate in khz
+ */
+MS2_PUBLIC int ms_snd_card_get_preferred_sample_rate(const MSSndCard *obj);
+
+/**
+ * set preferred sample rate. The underlying card will try to avoid any resampling for this samplerate.
+ *
+ * @param obj      A sound card object.
+ * @param rate     sampling rate. 
+ *
+ * Returns:  0 if successfull, <0 otherwise.
+ */
+MS2_PUBLIC int ms_snd_card_set_preferred_sample_rate(MSSndCard *obj,int rate);
+	
+
 /**
  * Create a alsa card with user supplied pcm name and mixer name.
  * @param pcmdev The pcm device name following alsa conventions (ex: plughw:0)
