@@ -40,11 +40,18 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 
 /**
- * Structure for method getting time in miliseconds from an external source.
+ * Function pointer for method getting time in miliseconds from an external source.
  * @var MSTickerTimeFunc
  */
 typedef uint64_t (*MSTickerTimeFunc)(void *);
 
+/**
+ * Function pointer for method waiting next tick from an external source.
+ * @var MSTickerTickFunc
+ * It shall return the number of late milliseconds, if this value is known.
+ */
+typedef int (*MSTickerTickFunc)(void *, uint64_t ticker_virtual_time);
+
 /**
  * Enum for ticker priority
 **/
@@ -61,6 +68,7 @@ struct _MSTicker
 	ms_mutex_t lock;
 	ms_cond_t cond;
 	MSList *execution_list;     /* the list of source filters to be executed.*/
+	MSList *task_list; /* list of tasks (see ms_filter_postpone_task())*/
 	ms_thread_t thread;   /* the thread ressource*/
 	int interval; /* in miliseconds*/
 	int exec_id;
@@ -72,6 +80,8 @@ struct _MSTicker
 	char *name;
 	double av_load;	/*average load of the ticker */
 	MSTickerPrio prio;
+	MSTickerTickFunc wait_next_tick;
+	void *wait_next_tick_data;
 	bool_t run;       /* flag to indicate whether the ticker must be run or not */
 };
 
@@ -82,6 +92,26 @@ struct _MSTicker
 typedef struct _MSTicker MSTicker;
 
 
+struct _MSTickerParams{
+	MSTickerPrio prio;
+	const char *name;
+};
+
+typedef struct _MSTickerParams MSTickerParams;
+
+
+struct _MSTickerSynchronizer
+{
+	uint64_t offset; /**<the default offset of ticker*/
+	double av_skew; /**< mean skew */
+};
+
+/**
+ * Structure for ticker synchronizer object.
+ * @var MSTickerSynchronizer
+ */
+typedef struct _MSTickerSynchronizer MSTickerSynchronizer;
+
 #ifdef __cplusplus
 extern "C"{
 #endif
@@ -95,13 +125,21 @@ extern "C"{
  */
 MS2_PUBLIC MSTicker *ms_ticker_new(void);
 
+/**
+ * Create a ticker that will be used to start
+ * and stop a graph.
+ *
+ * Returns: MSTicker * if successfull, NULL otherwise.
+ */
+MS2_PUBLIC MSTicker *ms_ticker_new_with_params(const MSTickerParams *params);
+	
 /**
  * Set a name to the ticker (used for logging)
 **/
 MS2_PUBLIC void ms_ticker_set_name(MSTicker *ticker, const char *name);
 
 /**
- * Set priority to the ticker
+ * Deprecated: Set priority to the ticker
 **/
 MS2_PUBLIC void ms_ticker_set_priority(MSTicker *ticker, MSTickerPrio prio);
 	
@@ -117,6 +155,19 @@ MS2_PUBLIC void ms_ticker_set_priority(MSTicker *ticker, MSTickerPrio prio);
  */
 MS2_PUBLIC int ms_ticker_attach(MSTicker *ticker,MSFilter *f);
 
+/**
+ * Attach a chain of filters to a ticker.
+ * The processing chain will be executed until ms_ticker_detach
+ * will be called.
+ * This variadic can be used to attach multiple chains in a single call. The argument list MUST be NULL terminated.
+ *
+ * @param ticker  A #MSTicker object.
+ * @param f       A #MSFilter object.
+ *
+ * Returns: 0 if successfull, -1 otherwise.
+ */
+MS2_PUBLIC int ms_ticker_attach_multiple(MSTicker *ticker,MSFilter *f,...);
+
 /**
  * Dettach a chain of filters to a ticker.
  * The processing chain will no more be executed.
@@ -141,6 +192,7 @@ MS2_PUBLIC void ms_ticker_destroy(MSTicker *ticker);
  * Override MSTicker's time function.
  * This can be used to control the ticker from an external time provider, for example the 
  * clock of a sound card.
+ * WARNING: this must not be used in conjunction with ms_ticker_set_tick_func().
  *
  * @param ticker  A #MSTicker object.
  * @param func    A replacement method for calculating "current time"
@@ -148,6 +200,17 @@ MS2_PUBLIC void ms_ticker_destroy(MSTicker *ticker);
  */
 MS2_PUBLIC void ms_ticker_set_time_func(MSTicker *ticker, MSTickerTimeFunc func, void *user_data);
 
+/**
+ * Override MSTicker's ticking function.
+ * This can be used to control the ticker from an external ticking source, for example an interrupt, an event on a file descriptor, etc.
+ * WARNING: this must not be used in conjunction with ms_ticker_set_time_func().
+ *
+ * @param ticker  A #MSTicker object.
+ * @param func    A replacement method waiting the next tick.
+ * @param user_data Any pointer to user private data.
+ */
+MS2_PUBLIC void ms_ticker_set_tick_func(MSTicker *ticker, MSTickerTickFunc func, void *user_data);
+
 /**
  * Print on stdout all filters of a ticker. (INTERNAL: DO NOT USE)
  *
@@ -163,7 +226,42 @@ MS2_PUBLIC void ms_ticker_print_graphs(MSTicker *ticker);
  * A load greater than 100% clearly means that the ticker is over loaded and runs late.
 **/
 MS2_PUBLIC float ms_ticker_get_average_load(MSTicker *ticker);
-	
+
+/**
+ * Create a ticker synchronizer.
+ *
+ * Returns: MSTickerSynchronizer * if successfull, NULL otherwise.
+ */
+MS2_PUBLIC MSTickerSynchronizer* ms_ticker_synchronizer_new(void);
+
+/**
+ * Set the current external time.
+ *
+ * @param ts  A #MSTickerSynchronizer object.
+ * @param time  A #MSTimeSpec object.
+ *
+ * Returns: Average skew.
+ */
+MS2_PUBLIC double ms_ticker_synchronizer_set_external_time(MSTickerSynchronizer* ts, const MSTimeSpec *time);
+
+/**
+ * Get the corrected current time following the set external times.
+ *
+ * @param ts  A #MSTickerSynchronizer object.
+ *
+ *
+ * Returns: A corrected current time.
+ */
+MS2_PUBLIC uint64_t ms_ticker_synchronizer_get_corrected_time(MSTickerSynchronizer* ts);
+
+/**
+ * Destroy a ticker synchronizer.
+ *
+ * @param ts  A #MSTickerSynchronizer object.
+ *
+ */
+MS2_PUBLIC void ms_ticker_synchronizer_destroy(MSTickerSynchronizer* ts);
+
 /* private functions:*/
 
 #ifdef __cplusplus
