@@ -188,6 +188,7 @@ typedef enum{
 	MS_YUY2,   /* -> same as MS_YUYV */
 	MS_RGBA32,
 	MS_RGB565,
+	MS_H264,
 	MS_PIX_FMT_UNKNOWN
 }MSPixFmt;
 
@@ -211,6 +212,9 @@ MS2_PUBLIC int ms_yuv_buf_init_from_mblk(MSPicture *buf, mblk_t *m);
 MS2_PUBLIC int ms_yuv_buf_init_from_mblk_with_size(MSPicture *buf, mblk_t *m, int w, int h);
 MS2_PUBLIC int ms_picture_init_from_mblk_with_size(MSPicture *buf, mblk_t *m, MSPixFmt fmt, int w, int h);
 MS2_PUBLIC mblk_t * ms_yuv_buf_alloc(MSPicture *buf, int w, int h);
+
+/* Allocates a video mblk_t with supplied width and height, the pixels being contained in an external buffer.
+The returned mblk_t points to the external buffer, which is not copied, nor ref'd: the reference is simply transfered to the returned mblk_t*/
 MS2_PUBLIC mblk_t * ms_yuv_buf_alloc_from_buffer(int w, int h, mblk_t* buffer);
 MS2_PUBLIC void ms_yuv_buf_copy(uint8_t *src_planes[], const int src_strides[],
 		uint8_t *dst_planes[], const int dst_strides[3], MSVideoSize roi);
@@ -319,11 +323,11 @@ MS2_PUBLIC bool_t ms_video_capture_new_frame(MSFrameRateController* ctrl, uint32
 struct _MSAverageFPS {
 	unsigned int last_frame_time, last_print_time;
 	float mean_inter_frame;
-	float expected_fps;
+	const char* context;
 };
 typedef struct _MSAverageFPS MSAverageFPS;
-MS2_PUBLIC void ms_video_init_average_fps(MSAverageFPS* afps, float expectedFps);
-MS2_PUBLIC void ms_video_update_average_fps(MSAverageFPS* afps, uint32_t current_time);
+MS2_PUBLIC void ms_video_init_average_fps(MSAverageFPS* afps, const char* context);
+MS2_PUBLIC bool_t ms_video_update_average_fps(MSAverageFPS* afps, uint32_t current_time);
 
 #ifdef __cplusplus
 }
@@ -339,6 +343,7 @@ MS2_PUBLIC void ms_video_update_average_fps(MSAverageFPS* afps, uint32_t current
 #define MS_FILTER_GET_FPS		MS_FILTER_BASE_METHOD(105,float)
 
 /* request a video-fast-update (=I frame for H263,MP4V-ES) to a video encoder*/
+/* DEPRECATED: Use MS_VIDEO_ENCODER_REQ_VFU instead */
 #define MS_FILTER_REQ_VFU		MS_FILTER_BASE_METHOD_NO_ARG(106)
 
 #endif
