@@ -34,6 +34,7 @@
 #include <ortp/port.h>
 #include <ortp/rtp.h>
 #include <ortp/payloadtype.h>
+#include <ortp/rtpprofile.h>
 #include <ortp/sessionset.h>
 #include <ortp/rtcp.h>
 #include <ortp/str_utils.h>
@@ -62,7 +63,7 @@ typedef struct _JBParameters{
 
 typedef struct _JitterControl
 {
-	int count;
+	unsigned int count;
 	int jitt_comp;   /* the user jitt_comp in miliseconds*/
 	int jitt_comp_ts; /* the jitt_comp converted in rtp time (same unit as timestamp) */
 	int adapt_jitt_comp_ts;
@@ -73,6 +74,9 @@ typedef struct _JitterControl
 	float inter_jitter;	/* interarrival jitter as defined in the RFC */
 	int corrective_step;
 	int corrective_slide;
+	uint64_t cum_jitter_buffer_size; /*in timestamp units*/
+	unsigned int cum_jitter_buffer_count; /*used for computation of jitter buffer size*/
+	int clock_rate;
 	bool_t adaptive;
 	bool_t enabled;
 } JitterControl;
@@ -167,6 +171,7 @@ typedef struct _RtcpStream
 	int sockfamily;
 	struct _RtpTransport *tr; 
 	mblk_t *cached_mp;
+	int loc_port;
 #ifdef ORTP_INET6
 	struct sockaddr_storage rem_addr;
 #else
@@ -242,6 +247,7 @@ struct _RtpSession
 	bool_t permissive; /*use the permissive algorithm*/
 	bool_t use_connect; /* use connect() on the socket */
 	bool_t ssrc_set;
+	bool_t reuseaddr; /*setsockopt SO_REUSEADDR */
 };
 	
 
@@ -253,163 +259,171 @@ extern "C"
 #endif
 
 /* public API */
-RtpSession *rtp_session_new(int mode);
-void rtp_session_set_scheduling_mode(RtpSession *session, int yesno);
-void rtp_session_set_blocking_mode(RtpSession *session, int yesno);
-void rtp_session_set_profile(RtpSession *session, RtpProfile *profile);
-void rtp_session_set_send_profile(RtpSession *session,RtpProfile *profile);
-void rtp_session_set_recv_profile(RtpSession *session,RtpProfile *profile);
-RtpProfile *rtp_session_get_profile(RtpSession *session);
-RtpProfile *rtp_session_get_send_profile(RtpSession *session);
-RtpProfile *rtp_session_get_recv_profile(RtpSession *session);
-int rtp_session_signal_connect(RtpSession *session,const char *signal_name, RtpCallback cb, unsigned long user_data);
-int rtp_session_signal_disconnect_by_callback(RtpSession *session,const char *signal_name, RtpCallback cb);
-void rtp_session_set_ssrc(RtpSession *session, uint32_t ssrc);
-uint32_t rtp_session_get_send_ssrc(RtpSession* session);
-void rtp_session_set_seq_number(RtpSession *session, uint16_t seq);
-uint16_t rtp_session_get_seq_number(RtpSession *session);
+ORTP_PUBLIC RtpSession *rtp_session_new(int mode);
+ORTP_PUBLIC void rtp_session_set_scheduling_mode(RtpSession *session, int yesno);
+ORTP_PUBLIC void rtp_session_set_blocking_mode(RtpSession *session, int yesno);
+ORTP_PUBLIC void rtp_session_set_profile(RtpSession *session, RtpProfile *profile);
+ORTP_PUBLIC void rtp_session_set_send_profile(RtpSession *session,RtpProfile *profile);
+ORTP_PUBLIC void rtp_session_set_recv_profile(RtpSession *session,RtpProfile *profile);
+ORTP_PUBLIC RtpProfile *rtp_session_get_profile(RtpSession *session);
+ORTP_PUBLIC RtpProfile *rtp_session_get_send_profile(RtpSession *session);
+ORTP_PUBLIC RtpProfile *rtp_session_get_recv_profile(RtpSession *session);
+ORTP_PUBLIC int rtp_session_signal_connect(RtpSession *session,const char *signal_name, RtpCallback cb, unsigned long user_data);
+ORTP_PUBLIC int rtp_session_signal_disconnect_by_callback(RtpSession *session,const char *signal_name, RtpCallback cb);
+ORTP_PUBLIC void rtp_session_set_ssrc(RtpSession *session, uint32_t ssrc);
+ORTP_PUBLIC uint32_t rtp_session_get_send_ssrc(RtpSession* session);
+ORTP_PUBLIC uint32_t rtp_session_get_recv_ssrc(RtpSession *session);
+ORTP_PUBLIC void rtp_session_set_seq_number(RtpSession *session, uint16_t seq);
+ORTP_PUBLIC uint16_t rtp_session_get_seq_number(RtpSession *session);
+ORTP_PUBLIC uint32_t rtp_session_get_rcv_ext_seq_number(RtpSession *session);
 
-void rtp_session_enable_jitter_buffer(RtpSession *session , bool_t enabled);
-bool_t rtp_session_jitter_buffer_enabled(const RtpSession *session);
-void rtp_session_set_jitter_buffer_params(RtpSession *session, const JBParameters *par);
-void rtp_session_get_jitter_buffer_params(RtpSession *session, JBParameters *par);
+ORTP_PUBLIC void rtp_session_enable_jitter_buffer(RtpSession *session , bool_t enabled);
+ORTP_PUBLIC bool_t rtp_session_jitter_buffer_enabled(const RtpSession *session);
+ORTP_PUBLIC void rtp_session_set_jitter_buffer_params(RtpSession *session, const JBParameters *par);
+ORTP_PUBLIC void rtp_session_get_jitter_buffer_params(RtpSession *session, JBParameters *par);
 
 /*deprecated jitter control functions*/
-void rtp_session_set_jitter_compensation(RtpSession *session, int milisec);
-void rtp_session_enable_adaptive_jitter_compensation(RtpSession *session, bool_t val);
-bool_t rtp_session_adaptive_jitter_compensation_enabled(RtpSession *session);
+ORTP_PUBLIC void rtp_session_set_jitter_compensation(RtpSession *session, int milisec);
+ORTP_PUBLIC void rtp_session_enable_adaptive_jitter_compensation(RtpSession *session, bool_t val);
+ORTP_PUBLIC bool_t rtp_session_adaptive_jitter_compensation_enabled(RtpSession *session);
 
-void rtp_session_set_time_jump_limit(RtpSession *session, int miliseconds);
-int rtp_session_set_local_addr(RtpSession *session,const char *addr, int port);
-int rtp_session_get_local_port(const RtpSession *session);
+ORTP_PUBLIC void rtp_session_set_time_jump_limit(RtpSession *session, int miliseconds);
+ORTP_PUBLIC int rtp_session_set_local_addr(RtpSession *session,const char *addr, int rtp_port, int rtcp_port);
+ORTP_PUBLIC int rtp_session_get_local_port(const RtpSession *session);
 
-int
-rtp_session_set_remote_addr_full (RtpSession * session, const char * addr, int rtp_port, int rtcp_port);
+ORTP_PUBLIC int
+rtp_session_set_remote_addr_full (RtpSession * session, const char * rtp_addr, int rtp_port, const char * rtcp_addr, int rtcp_port);
 /*same as previous function, old name:*/
-int rtp_session_set_remote_addr_and_port (RtpSession * session, const char * addr, int rtp_port, int rtcp_port);
-int rtp_session_set_remote_addr(RtpSession *session,const char *addr, int port);
+ORTP_PUBLIC int rtp_session_set_remote_addr_and_port (RtpSession * session, const char * addr, int rtp_port, int rtcp_port);
+ORTP_PUBLIC int rtp_session_set_remote_addr(RtpSession *session,const char *addr, int port);
 /* alternatively to the set_remote_addr() and set_local_addr(), an application can give
 a valid socket (potentially connect()ed )to be used by the RtpSession */
-void rtp_session_set_sockets(RtpSession *session, int rtpfd, int rtcpfd);
-void rtp_session_set_transports(RtpSession *session, RtpTransport *rtptr, RtpTransport *rtcptr);
+ORTP_PUBLIC void rtp_session_set_sockets(RtpSession *session, int rtpfd, int rtcpfd);
+ORTP_PUBLIC void rtp_session_set_transports(RtpSession *session, RtpTransport *rtptr, RtpTransport *rtcptr);
 
 /*those methods are provided for people who wants to send non-RTP messages using the RTP/RTCP sockets */
-ortp_socket_t rtp_session_get_rtp_socket(const RtpSession *session);
-ortp_socket_t rtp_session_get_rtcp_socket(const RtpSession *session);
+ORTP_PUBLIC ortp_socket_t rtp_session_get_rtp_socket(const RtpSession *session);
+ORTP_PUBLIC ortp_socket_t rtp_session_get_rtcp_socket(const RtpSession *session);
 
 
 /* QOS / DSCP */
-int rtp_session_set_dscp(RtpSession *session, int dscp);
-int rtp_session_get_dscp(const RtpSession *session);
+ORTP_PUBLIC int rtp_session_set_dscp(RtpSession *session, int dscp);
+ORTP_PUBLIC int rtp_session_get_dscp(const RtpSession *session);
 
 
+/* Packet info */
+ORTP_PUBLIC int rtp_session_set_pktinfo(RtpSession *session, int activate);
+
 /* Multicast methods */
-int rtp_session_set_multicast_ttl(RtpSession *session, int ttl);
-int rtp_session_get_multicast_ttl(RtpSession *session);
+ORTP_PUBLIC int rtp_session_set_multicast_ttl(RtpSession *session, int ttl);
+ORTP_PUBLIC int rtp_session_get_multicast_ttl(RtpSession *session);
 
-int rtp_session_set_multicast_loopback(RtpSession *session, int yesno);
-int rtp_session_get_multicast_loopback(RtpSession *session);
+ORTP_PUBLIC int rtp_session_set_multicast_loopback(RtpSession *session, int yesno);
+ORTP_PUBLIC int rtp_session_get_multicast_loopback(RtpSession *session);
 
 
 
-int rtp_session_set_send_payload_type(RtpSession *session, int paytype);
-int rtp_session_get_send_payload_type(const RtpSession *session);
+ORTP_PUBLIC int rtp_session_set_send_payload_type(RtpSession *session, int paytype);
+ORTP_PUBLIC int rtp_session_get_send_payload_type(const RtpSession *session);
 
-int rtp_session_get_recv_payload_type(const RtpSession *session);
-int rtp_session_set_recv_payload_type(RtpSession *session, int pt);
+ORTP_PUBLIC int rtp_session_get_recv_payload_type(const RtpSession *session);
+ORTP_PUBLIC int rtp_session_set_recv_payload_type(RtpSession *session, int pt);
 
-int rtp_session_set_payload_type(RtpSession *session, int pt);
+ORTP_PUBLIC int rtp_session_set_payload_type(RtpSession *session, int pt);
 
-void rtp_session_set_symmetric_rtp (RtpSession * session, bool_t yesno);
+ORTP_PUBLIC void rtp_session_set_symmetric_rtp (RtpSession * session, bool_t yesno);
 
-void rtp_session_set_connected_mode(RtpSession *session, bool_t yesno);
+ORTP_PUBLIC void rtp_session_set_connected_mode(RtpSession *session, bool_t yesno);
 
-void rtp_session_enable_rtcp(RtpSession *session, bool_t yesno);
+ORTP_PUBLIC void rtp_session_enable_rtcp(RtpSession *session, bool_t yesno);
 
-void rtp_session_set_rtcp_report_interval(RtpSession *session, int value_ms);
+ORTP_PUBLIC void rtp_session_set_rtcp_report_interval(RtpSession *session, int value_ms);
 
-void rtp_session_set_ssrc_changed_threshold(RtpSession *session, int numpackets);
+ORTP_PUBLIC void rtp_session_set_ssrc_changed_threshold(RtpSession *session, int numpackets);
 
 /*low level recv and send functions */
-mblk_t * rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts);
-mblk_t * rtp_session_create_packet(RtpSession *session,int header_size, const uint8_t *payload, int payload_size);
-mblk_t * rtp_session_create_packet_with_data(RtpSession *session, uint8_t *payload, int payload_size, void (*freefn)(void*));
-mblk_t * rtp_session_create_packet_in_place(RtpSession *session,uint8_t *buffer, int size, void (*freefn)(void*) );
-int rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, uint32_t userts);
+ORTP_PUBLIC mblk_t * rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts);
+ORTP_PUBLIC mblk_t * rtp_session_create_packet(RtpSession *session,int header_size, const uint8_t *payload, int payload_size);
+ORTP_PUBLIC mblk_t * rtp_session_create_packet_with_data(RtpSession *session, uint8_t *payload, int payload_size, void (*freefn)(void*));
+ORTP_PUBLIC mblk_t * rtp_session_create_packet_in_place(RtpSession *session,uint8_t *buffer, int size, void (*freefn)(void*) );
+ORTP_PUBLIC int rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, uint32_t userts);
 /* high level recv and send functions */
-int rtp_session_recv_with_ts(RtpSession *session, uint8_t *buffer, int len, uint32_t ts, int *have_more);
-int rtp_session_send_with_ts(RtpSession *session, const uint8_t *buffer, int len, uint32_t userts);
+ORTP_PUBLIC int rtp_session_recv_with_ts(RtpSession *session, uint8_t *buffer, int len, uint32_t ts, int *have_more);
+ORTP_PUBLIC int rtp_session_send_with_ts(RtpSession *session, const uint8_t *buffer, int len, uint32_t userts);
 
 /* event API*/
-void rtp_session_register_event_queue(RtpSession *session, OrtpEvQueue *q);
-void rtp_session_unregister_event_queue(RtpSession *session, OrtpEvQueue *q);
+ORTP_PUBLIC void rtp_session_register_event_queue(RtpSession *session, OrtpEvQueue *q);
+ORTP_PUBLIC void rtp_session_unregister_event_queue(RtpSession *session, OrtpEvQueue *q);
 
 
 /* IP bandwidth usage estimation functions, returning bits/s*/
-float rtp_session_compute_send_bandwidth(RtpSession *session);
-float rtp_session_compute_recv_bandwidth(RtpSession *session);
+ORTP_PUBLIC float rtp_session_compute_send_bandwidth(RtpSession *session);
+ORTP_PUBLIC float rtp_session_compute_recv_bandwidth(RtpSession *session);
 
-void rtp_session_send_rtcp_APP(RtpSession *session, uint8_t subtype, const char *name, const uint8_t *data, int datalen);
+ORTP_PUBLIC void rtp_session_send_rtcp_APP(RtpSession *session, uint8_t subtype, const char *name, const uint8_t *data, int datalen);
 
-uint32_t rtp_session_get_current_send_ts(RtpSession *session);
-uint32_t rtp_session_get_current_recv_ts(RtpSession *session);
-void rtp_session_flush_sockets(RtpSession *session);
-void rtp_session_release_sockets(RtpSession *session);
-void rtp_session_resync(RtpSession *session);
-void rtp_session_reset(RtpSession *session);
-void rtp_session_destroy(RtpSession *session);
+ORTP_PUBLIC uint32_t rtp_session_get_current_send_ts(RtpSession *session);
+ORTP_PUBLIC uint32_t rtp_session_get_current_recv_ts(RtpSession *session);
+ORTP_PUBLIC void rtp_session_flush_sockets(RtpSession *session);
+ORTP_PUBLIC void rtp_session_release_sockets(RtpSession *session);
+ORTP_PUBLIC void rtp_session_resync(RtpSession *session);
+ORTP_PUBLIC void rtp_session_reset(RtpSession *session);
+ORTP_PUBLIC void rtp_session_destroy(RtpSession *session);
 
-const rtp_stats_t * rtp_session_get_stats(const RtpSession *session);
-const jitter_stats_t * rtp_session_get_jitter_stats( const RtpSession *session );
-void rtp_session_reset_stats(RtpSession *session);
+ORTP_PUBLIC const rtp_stats_t * rtp_session_get_stats(const RtpSession *session);
+ORTP_PUBLIC const jitter_stats_t * rtp_session_get_jitter_stats( const RtpSession *session );
+ORTP_PUBLIC void rtp_session_reset_stats(RtpSession *session);
 
-void rtp_session_set_data(RtpSession *session, void *data);
-void *rtp_session_get_data(const RtpSession *session);
+ORTP_PUBLIC void rtp_session_set_data(RtpSession *session, void *data);
+ORTP_PUBLIC void *rtp_session_get_data(const RtpSession *session);
 
-void rtp_session_set_recv_buf_size(RtpSession *session, int bufsize);
-void rtp_session_set_rtp_socket_send_buffer_size(RtpSession * session, unsigned int size);
-void rtp_session_set_rtp_socket_recv_buffer_size(RtpSession * session, unsigned int size);
+ORTP_PUBLIC void rtp_session_set_recv_buf_size(RtpSession *session, int bufsize);
+ORTP_PUBLIC void rtp_session_set_rtp_socket_send_buffer_size(RtpSession * session, unsigned int size);
+ORTP_PUBLIC void rtp_session_set_rtp_socket_recv_buffer_size(RtpSession * session, unsigned int size);
 
 /* in use with the scheduler to convert a timestamp in scheduler time unit (ms) */
-uint32_t rtp_session_ts_to_time(RtpSession *session,uint32_t timestamp);
-uint32_t rtp_session_time_to_ts(RtpSession *session, int millisecs);
+ORTP_PUBLIC uint32_t rtp_session_ts_to_time(RtpSession *session,uint32_t timestamp);
+ORTP_PUBLIC uint32_t rtp_session_time_to_ts(RtpSession *session, int millisecs);
 /* this function aims at simulating senders with "imprecise" clocks, resulting in 
 rtp packets sent with timestamp uncorrelated with the system clock .
 This is only availlable to sessions working with the oRTP scheduler */
-void rtp_session_make_time_distorsion(RtpSession *session, int milisec);
+ORTP_PUBLIC void rtp_session_make_time_distorsion(RtpSession *session, int milisec);
 
 /*RTCP functions */
-void rtp_session_set_source_description(RtpSession *session, const char *cname,
+ORTP_PUBLIC void rtp_session_set_source_description(RtpSession *session, const char *cname,
 	const char *name, const char *email, const char *phone, 
     const char *loc, const char *tool, const char *note);
-void rtp_session_add_contributing_source(RtpSession *session, uint32_t csrc, 
+ORTP_PUBLIC void rtp_session_add_contributing_source(RtpSession *session, uint32_t csrc, 
     const char *cname, const char *name, const char *email, const char *phone, 
     const char *loc, const char *tool, const char *note);
-void rtp_session_remove_contributing_sources(RtpSession *session, uint32_t csrc);
-mblk_t* rtp_session_create_rtcp_sdes_packet(RtpSession *session);
+ORTP_PUBLIC void rtp_session_remove_contributing_sources(RtpSession *session, uint32_t csrc);
+ORTP_PUBLIC mblk_t* rtp_session_create_rtcp_sdes_packet(RtpSession *session);
 
-void rtp_session_get_last_recv_time(RtpSession *session, struct timeval *tv);
-int rtp_session_bye(RtpSession *session, const char *reason);
+ORTP_PUBLIC void rtp_session_get_last_recv_time(RtpSession *session, struct timeval *tv);
+ORTP_PUBLIC int rtp_session_bye(RtpSession *session, const char *reason);
 
-int rtp_session_get_last_send_error_code(RtpSession *session);
-void rtp_session_clear_send_error_code(RtpSession *session);
-int rtp_session_get_last_recv_error_code(RtpSession *session);
-void rtp_session_clear_recv_error_code(RtpSession *session);
+ORTP_PUBLIC int rtp_session_get_last_send_error_code(RtpSession *session);
+ORTP_PUBLIC void rtp_session_clear_send_error_code(RtpSession *session);
+ORTP_PUBLIC int rtp_session_get_last_recv_error_code(RtpSession *session);
+ORTP_PUBLIC void rtp_session_clear_recv_error_code(RtpSession *session);
 
 
-float rtp_session_get_round_trip_propagation(RtpSession *session);
+ORTP_PUBLIC float rtp_session_get_round_trip_propagation(RtpSession *session);
 
 
-void rtp_session_enable_network_simulation(RtpSession *session, const OrtpNetworkSimulatorParams *params);
-void rtp_session_rtcp_set_lost_packet_value( RtpSession *session, const unsigned int value );
-void rtp_session_rtcp_set_jitter_value(RtpSession *session, const unsigned int value );
-void rtp_session_rtcp_set_delay_value(RtpSession *session, const unsigned int value );
-mblk_t * rtp_session_pick_with_cseq (RtpSession * session, const uint16_t sequence_number);
+ORTP_PUBLIC void rtp_session_enable_network_simulation(RtpSession *session, const OrtpNetworkSimulatorParams *params);
+ORTP_PUBLIC void rtp_session_rtcp_set_lost_packet_value( RtpSession *session, const unsigned int value );
+ORTP_PUBLIC void rtp_session_rtcp_set_jitter_value(RtpSession *session, const unsigned int value );
+ORTP_PUBLIC void rtp_session_rtcp_set_delay_value(RtpSession *session, const unsigned int value );
+ORTP_PUBLIC mblk_t * rtp_session_pick_with_cseq (RtpSession * session, const uint16_t sequence_number);
 /*private */
-void rtp_session_init(RtpSession *session, int mode);
+ORTP_PUBLIC void rtp_session_init(RtpSession *session, int mode);
 #define rtp_session_set_flag(session,flag) (session)->flags|=(flag)
 #define rtp_session_unset_flag(session,flag) (session)->flags&=~(flag)
-void rtp_session_uninit(RtpSession *session);
+ORTP_PUBLIC void rtp_session_uninit(RtpSession *session);
+ORTP_PUBLIC void rtp_session_dispatch_event(RtpSession *session, OrtpEvent *ev);
+
+ORTP_PUBLIC void rtp_session_set_reuseaddr(RtpSession *session, bool_t yes);
 
 #ifdef __cplusplus
 }
