@@ -76,6 +76,9 @@
 #include <ortp/port.h>
 #include <ortp/stun_udp.h>
 
+#ifdef __APPLE__
+   #include "TargetConditionals.h"
+#endif
 
 #ifdef __cplusplus
 extern "C"
@@ -83,24 +86,27 @@ extern "C"
 #endif
 
 /* if you change this version, change in makefile too  */
-#define STUN_VERSION "0.94"
+#define STUN_VERSION "0.99"
 
-#define STUN_MAX_STRING 256
+#define STUN_MAX_STRING 514
 #define STUN_MAX_UNKNOWN_ATTRIBUTES 8
 #define STUN_MAX_MESSAGE_SIZE 2048
 
 #define STUN_PORT 3478
 
 /* define some basic types */
-typedef unsigned char  UInt8;
-typedef unsigned short UInt16;
-typedef unsigned int   UInt32;
+#if 0
+typedef unsigned char  uint8_t;
+typedef unsigned short uint16_t;
+typedef unsigned int   uint32_t;
+
 #if	defined(WIN32) || defined(_WIN32_WCE)
-typedef unsigned __int64 UInt64;
+typedef unsigned __int64 uint64_t;
 #else
-typedef unsigned long long UInt64;
+typedef unsigned long long uint64_t;
 #endif
-typedef struct { unsigned char octet[16]; }  UInt128;
+#endif
+typedef struct { unsigned char octet[12]; }  UInt96;
 
 /* define a structure to hold a stun address  */
 #define  IPv4Family  0x01
@@ -111,83 +117,173 @@ typedef struct { unsigned char octet[16]; }  UInt128;
 #define ChangePortFlag  0x02
 
 /* define  stun attribute */
-#define MappedAddress     0x0001
-#define ResponseAddress   0x0002
-#define ChangeRequest     0x0003
-#define SourceAddress     0x0004
-#define ChangedAddress    0x0005
-#define STUNUsername      0x0006 /* Username is too common: rename to avoid conflict */
-#define STUNPassword      0x0007 /* Password is too common: rename to avoid conflict */
-#define MessageIntegrity  0x0008
-#define ErrorCode         0x0009
-#define UnknownAttribute  0x000A
-#define ReflectedFrom     0x000B
-#define XorMappedAddress  0x0020
-#define XorOnly           0x0021
-#define ServerName        0x0022
-#define SecondaryAddress  0x0050 /* Non standard extention */
+#define SA_MAPPEDADDRESS     0x0001
+#define SA_RESPONSEADDRESS   0x0002 /** deprecated **/
+#define SA_CHANGEREQUEST     0x0003 /** deprecated **/
+#define SA_SOURCEADDRESS     0x0004 /** deprecated **/
+#define SA_CHANGEDADDRESS    0x0005 /** deprecated **/
+#define SA_USERNAME          0x0006
+#define SA_PASSWORD          0x0007  /** deprecated **/
+#define SA_MESSAGEINTEGRITY  0x0008
+#define SA_ERRORCODE         0x0009
+#define SA_UNKNOWNATTRIBUTE  0x000A
+#define SA_REFLECTEDFROM     0x000B /** deprecated **/
+#define SA_REALM             0x0014
+#define SA_NONCE             0x0015
+#define SA_XORMAPPEDADDRESS  0x0020
+
+#define SA_XORMAPPEDADDRESS2 0x8020 /* Non standard extention */
+#define SA_XORONLY           0x0021 /* deprecated */
+#define SA_SECONDARYADDRESS  0x0050 /* Non standard extention */
+
+#define SA_SOFTWARE          0x8022
+#define SA_ALTERNATESERVER   0x8023
+#define SA_FINGERPRINT       0x8028
+
+/* define turn attribute */
+#define TA_CHANNELNUMBER       0x000C
+#define TA_LIFETIME            0x000D
+#define TA_DEPRECATEDBANDWIDTH 0x0010
+#define TA_XORPEERADDRESS      0x0012
+#define TA_DATA                0x0013
+#define TA_XORRELAYEDADDRESS   0x0016
+#define TA_EVENPORT            0x0018
+#define TA_REQUESTEDTRANSPORT  0x0019
+#define TA_DONTFRAGMENT        0x001A
+#define TA_DEPRECATEDTIMERVAL  0x0021
+#define TA_RESERVATIONTOKEN    0x0022
+
+#define ICEA_PRIORITY          0x0024
+#define ICEA_USECANDIDATE      0x0025
+#define ICEA_ICECONTROLLED     0x8029
+#define ICEA_ICECONTROLLING    0x802a
+
+#define STUN_REQUEST           0x0000
+#define STUN_INDICATION        0x0010
+#define STUN_SUCCESS_RESP      0x0100
+#define STUN_ERR_RESP          0x0110
+
+#define STUN_IS_REQUEST(msg_type)       (((msg_type) & 0x0110) == 0x0000)
+#define STUN_IS_INDICATION(msg_type)    (((msg_type) & 0x0110) == 0x0010)
+#define STUN_IS_SUCCESS_RESP(msg_type)  (((msg_type) & 0x0110) == 0x0100)
+#define STUN_IS_ERR_RESP(msg_type)      (((msg_type) & 0x0110) == 0x0110)
 
 /* define types for a stun message */
-#define BindRequestMsg                0x0001
-#define BindResponseMsg               0x0101
-#define BindErrorResponseMsg          0x0111
+#define STUN_METHOD_BINDING           0x0001
+#define TURN_MEDHOD_ALLOCATE          0x0003 //(only request/response semantics defined)
+#define TURN_METHOD_REFRESH           0x0004 //(only request/response semantics defined)
+#define TURN_METHOD_CREATEPERMISSION  0x0008 //(only request/response semantics defined
+#define TURN_METHOD_CHANNELBIND       0x0009 //(only request/response semantics defined)
+
+//#define BindResponseMsg               0x0101
+//#define BindErrorResponseMsg          0x0111
 #define SharedSecretRequestMsg        0x0002
 #define SharedSecretResponseMsg       0x0102
 #define SharedSecretErrorResponseMsg  0x0112
 
+#define TURN_INDICATION_SEND          0x0006 //(only indication semantics defined)
+#define TURN_INDICATION_DATA          0x0007 //(only indication semantics defined)
+
 typedef struct 
 {
-      UInt16 msgType;
-      UInt16 msgLength;
-      UInt128 id;
+      uint16_t msgType;
+      uint16_t msgLength;
+      uint32_t magic_cookie;
+      UInt96 tr_id;
 } StunMsgHdr;
 
 
 typedef struct
 {
-      UInt16 type;
-      UInt16 length;
+      uint16_t type;
+      uint16_t length;
 } StunAtrHdr;
 
 typedef struct
 {
-      UInt16 port;
-      UInt32 addr;
+      uint16_t port;
+      uint32_t addr;
 } StunAddress4;
 
 typedef struct
 {
-      UInt8 pad;
-      UInt8 family;
+      uint8_t pad;
+      uint8_t family;
       StunAddress4 ipv4;
 } StunAtrAddress4;
 
 typedef struct
 {
-      UInt32 value;
+      uint32_t value;
 } StunAtrChangeRequest;
 
 typedef struct
 {
-      UInt16 pad; /* all 0 */
-      UInt8 errorClass;
-      UInt8 number;
+      uint16_t pad; /* all 0 */
+      uint8_t errorClass;
+      uint8_t number;
       char reason[STUN_MAX_STRING];
-      UInt16 sizeReason;
+      uint16_t sizeReason;
 } StunAtrError;
 
 typedef struct
 {
-      UInt16 attrType[STUN_MAX_UNKNOWN_ATTRIBUTES];
-      UInt16 numAttributes;
+      uint16_t attrType[STUN_MAX_UNKNOWN_ATTRIBUTES];
+      uint16_t numAttributes;
 } StunAtrUnknown;
 
+typedef struct
+{
+      uint16_t channelNumber;
+      uint16_t rffu; /* Reserved For Future Use */
+} TurnAtrChannelNumber;
+
+typedef struct
+{
+      uint32_t lifetime;
+} TurnAtrLifetime;
+
+typedef struct
+{
+      char value[1500];      
+      uint16_t sizeValue;
+} TurnAtrData;
+
+typedef struct
+{
+      uint8_t proto;
+      uint8_t pad1;
+      uint8_t pad2;
+      uint8_t pad3;
+} TurnAtrRequestedTransport;
+
+typedef struct
+{
+      uint64_t value;
+} TurnAtrReservationToken;
+
+typedef struct
+{
+      uint32_t fingerprint;
+} StunAtrFingerprint;
+
+
 typedef struct
 {
       char value[STUN_MAX_STRING];      
-      UInt16 sizeValue;
+      uint16_t sizeValue;
 } StunAtrString;
 
+typedef struct
+{
+      uint32_t priority;
+} IceAtrPriority;
+
+typedef struct
+{
+      uint64_t value;
+} IceAtrIceControll;
+
 typedef struct
 {
       char hash[20];
@@ -202,6 +298,13 @@ typedef enum
    HmacFailed
 } StunHmacStatus;
 
+
+typedef struct
+{
+      uint16_t attrType[STUN_MAX_UNKNOWN_ATTRIBUTES];
+      uint16_t numAttributes;
+} TurnAtrUnknown;
+
 typedef struct
 {
       StunMsgHdr msgHdr;
@@ -239,16 +342,55 @@ typedef struct
       bool_t hasReflectedFrom;
       StunAtrAddress4 reflectedFrom;
 
+      bool_t hasRealm;
+      StunAtrString realmName;
+
+      bool_t hasNonce;
+      StunAtrString nonceName;
+
       bool_t hasXorMappedAddress;
       StunAtrAddress4  xorMappedAddress;
 	
-      bool_t xorOnly;
+      bool_t hasSoftware;
+      StunAtrString softwareName;
 
-      bool_t hasServerName;
-      StunAtrString serverName;
-      
-      bool_t hasSecondaryAddress;
-      StunAtrAddress4 secondaryAddress;
+      bool_t hasXorPeerAddress;
+      StunAtrAddress4 xorPeerAddress;
+
+      bool_t hasXorRelayedAddress;
+      StunAtrAddress4 xorRelayedAddress;
+
+      bool_t hasFingerprint;
+      StunAtrFingerprint fingerprint;
+
+      /* Turn elements */
+      bool_t hasChannelNumberAttributes;
+      TurnAtrChannelNumber channelNumberAttributes;
+
+      bool_t hasLifetimeAttributes;
+      TurnAtrLifetime lifetimeAttributes;
+
+      bool_t hasData;
+      TurnAtrData data;
+
+      bool_t hasRequestedTransport;
+      TurnAtrRequestedTransport requestedTransport;
+
+      bool_t hasDontFragment;
+
+      bool_t hasReservationToken;
+      TurnAtrReservationToken reservationToken;
+
+      bool_t hasPriority;
+      IceAtrPriority priority;
+
+      bool_t hasUseCandidate;
+
+      bool_t hasIceControlled;
+      IceAtrIceControll iceControlled;
+
+      bool_t hasIceControlling;
+      IceAtrIceControll iceControlling;
 } StunMessage; 
 
 
@@ -291,112 +433,114 @@ typedef struct
       StunMediaRelay relays[MAX_MEDIA_RELAYS];
 } StunServerInfo;
 
-bool_t
+ORTP_PUBLIC void
+stunCalculateIntegrity_longterm(char* hmac, const char* input, int length,
+                     const char *username, const char *realm, const char *password);
+ORTP_PUBLIC void
+stunCalculateIntegrity_shortterm(char* hmac, const char* input, int length, const char* key);
+ORTP_PUBLIC uint32_t
+stunCalculateFingerprint(const char* input, int length);
+
+ORTP_PUBLIC bool_t
 stunParseMessage( char* buf, 
                   unsigned int bufLen, 
-                  StunMessage *message, 
-                  bool_t verbose );
+                  StunMessage *message);
 
-void
+ORTP_PUBLIC void
 stunBuildReqSimple( StunMessage* msg,
                     const StunAtrString *username,
                     bool_t changePort, bool_t changeIp, unsigned int id );
 
-unsigned int
+ORTP_PUBLIC unsigned int
 stunEncodeMessage( const StunMessage *message, 
                    char* buf, 
                    unsigned int bufLen, 
-                   const StunAtrString *password,
-                   bool_t verbose);
+                   const StunAtrString *password);
 
-void
+ORTP_PUBLIC void
 stunCreateUserName(const StunAddress4 *addr, StunAtrString* username);
 
-void 
+ORTP_PUBLIC void
 stunGetUserNameAndPassword(  const StunAddress4 *dest, 
                              StunAtrString* username,
                              StunAtrString* password);
 
-void
+ORTP_PUBLIC void
 stunCreatePassword(const StunAtrString *username, StunAtrString* password);
 
-int 
+ORTP_PUBLIC int 
 stunRand(void);
 
-UInt64
+ORTP_PUBLIC uint64_t
 stunGetSystemTimeSecs(void);
 
 /* find the IP address of a the specified stun server - return false is fails parse  */
-bool_t  
-stunParseServerName( char* serverName, StunAddress4 *stunServerAddr);
+ORTP_PUBLIC bool_t  
+stunParseServerName( const char* serverName, StunAddress4 *stunServerAddr);
 
-bool_t 
-stunParseHostName( char* peerName,
-                   UInt32 *ip,
-                   UInt16 *portVal,
-                   UInt16 defaultPort );
+ORTP_PUBLIC bool_t 
+stunParseHostName( const char* peerName,
+                   uint32_t *ip,
+                   uint16_t *portVal,
+                   uint16_t defaultPort );
 
 /* return true if all is OK 
    Create a media relay and do the STERN thing if startMediaPort is non-zero */
-bool_t
+ORTP_PUBLIC bool_t
 stunInitServer(StunServerInfo *info, 
                const StunAddress4 *myAddr, 
                const StunAddress4 *altAddr,
-               int startMediaPort,
-               bool_t verbose);
+               int startMediaPort);
 
-void
+ORTP_PUBLIC void
 stunStopServer(StunServerInfo *info);
 
-#if 0 /* no usefull here */
-/* return true if all is OK */
-bool_t
-stunServerProcess(StunServerInfo *info, bool_t verbose);
-#endif
-
 /* returns number of address found - take array or addres */
-int 
-stunFindLocalInterfaces(UInt32* addresses, int maxSize );
+ORTP_PUBLIC int
+stunFindLocalInterfaces(uint32_t* addresses, int maxSize );
 
-int 
-stunTest( StunAddress4 *dest, int testNum, bool_t verbose, StunAddress4* srcAddr, StunAddress4 *sMappedAddr, StunAddress4* sChangedAddr);
+ORTP_PUBLIC int 
+stunTest( StunAddress4 *dest, int testNum, StunAddress4* srcAddr, StunAddress4 *sMappedAddr, StunAddress4* sChangedAddr);
 
-NatType
-stunNatType( StunAddress4 *dest, bool_t verbose, 
+ORTP_PUBLIC NatType
+stunNatType( StunAddress4 *dest, 
              bool_t* preservePort, /* if set, is return for if NAT preservers ports or not */
              bool_t* hairpin ,  /* if set, is the return for if NAT will hairpin packets */
              int port, /* port to use for the test, 0 to choose random port */
              StunAddress4* sAddr /* NIC to use */
    );
 
-bool_t
+ORTP_PUBLIC bool_t
 stunServerProcessMsg( char* buf,
                       unsigned int bufLen,
                       StunAddress4 *from, 
-                      StunAddress4 *secondary,
                       StunAddress4 *myAddr,
                       StunAddress4 *altAddr, 
                       StunMessage *resp,
                       StunAddress4 *destination,
                       StunAtrString *hmacPassword,
                       bool_t* changePort,
-                      bool_t* changeIp,
-                      bool_t verbose);
+                      bool_t* changeIp);
 
-int
+ORTP_PUBLIC int
 stunOpenSocket( StunAddress4 *dest, 
                 StunAddress4* mappedAddr, 
                 int port, 
-                StunAddress4* srcAddr, 
-                bool_t verbose );
+                StunAddress4* srcAddr);
 
-bool_t
+ORTP_PUBLIC bool_t
 stunOpenSocketPair(StunAddress4 *dest,
                    StunAddress4* mapAddr_rtp, 
                    StunAddress4* mapAddr_rtcp, 
                    int* fd1, int* fd2, 
-                   int srcPort,  StunAddress4* srcAddr,
-                   bool_t verbose);
+                   int srcPort,  StunAddress4* srcAddr);
+
+ORTP_PUBLIC bool_t
+turnAllocateSocketPair(StunAddress4 *dest,
+                   StunAddress4* mapAddr_rtp, 
+                   StunAddress4* mapAddr_rtcp, 
+                   int* fd1, int* fd2, 
+                   int srcPort,  StunAddress4* srcAddr);
 
 #ifdef __cplusplus
 }
