@@ -19,6 +19,7 @@
 
 
 #include <ortp/payloadtype.h>
+#include <ortp/rtpprofile.h>
 
 char offset127=127; 
 char offset0xD5=(char)0xD5; 
@@ -596,7 +597,7 @@ PayloadType payload_type_silk_nb={
 	BITS_PER_SAMPLE( 0),
 	ZERO_PATTERN(NULL),
 	PATTERN_LENGTH( 0),
-	NORMAL_BITRATE(10000),
+	NORMAL_BITRATE(13000),
 	MIME_TYPE ("SILK"),
 	CHANNELS(1)
 };
@@ -634,3 +635,35 @@ PayloadType payload_type_silk_swb={
 	CHANNELS(1)
 };
 
+PayloadType payload_type_aaceld_22k={
+	TYPE( PAYLOAD_AUDIO_PACKETIZED),
+	CLOCK_RATE(22050),
+	BITS_PER_SAMPLE( 0),
+	ZERO_PATTERN(NULL),
+	PATTERN_LENGTH( 0),
+	NORMAL_BITRATE(32000),
+	MIME_TYPE ("mpeg4-generic"),
+	CHANNELS(1)
+};
+
+PayloadType payload_type_aaceld_44k={
+	TYPE( PAYLOAD_AUDIO_PACKETIZED),
+	CLOCK_RATE(44100),
+	BITS_PER_SAMPLE( 0),
+	ZERO_PATTERN(NULL),
+	PATTERN_LENGTH( 0),
+	NORMAL_BITRATE(64000),
+	MIME_TYPE ("mpeg4-generic"),
+	CHANNELS(1)
+};
+
+PayloadType payload_type_opus = {
+	TYPE(PAYLOAD_AUDIO_PACKETIZED),
+	CLOCK_RATE(48000),
+	BITS_PER_SAMPLE(0),
+	ZERO_PATTERN(NULL),
+	PATTERN_LENGTH(0),
+	NORMAL_BITRATE(20000),
+	MIME_TYPE("opus"),
+	CHANNELS(1)
+};
