@@ -30,19 +30,20 @@
 #undef PACKAGE_VERSION
 
 #include "ortp/ortp_srtp.h"
+#include "rtpsession_priv.h"
 #ifdef HAVE_SRTP
 
 #include "ortp/b64.h"
 
-#define SRTP_PAD_BYTES 64 /*?? */
+#define SRTP_PAD_BYTES (SRTP_MAX_TRAILER_LEN + 4)
 
 static int  srtp_sendto(RtpTransport *t, mblk_t *m, int flags, const struct sockaddr *to, socklen_t tolen){
 	srtp_t srtp=(srtp_t)t->data;
 	int slen;
 	err_status_t err;
 	/* enlarge the buffer for srtp to write its data */
-	msgpullup(m,msgdsize(m)+SRTP_PAD_BYTES);
-	slen=m->b_wptr-m->b_rptr;
+	slen=msgdsize(m);
+	msgpullup(m,slen+SRTP_PAD_BYTES);
 	err=srtp_protect(srtp,m->b_rptr,&slen);
 	if (err==err_status_ok){
 		return sendto(t->session->rtp.socket,(void*)m->b_rptr,slen,flags,to,tolen);
@@ -55,7 +56,7 @@ static int srtp_recvfrom(RtpTransport *t, mblk_t *m, int flags, struct sockaddr
 	srtp_t srtp=(srtp_t)t->data;
 	int err;
 	int slen;
-	err=recvfrom(t->session->rtp.socket,(void*)m->b_wptr,m->b_datap->db_lim-m->b_datap->db_base,flags,from,fromlen);
+	err=rtp_session_rtp_recv_abstract(t->session->rtp.socket,m,flags,from,fromlen);
 	if (err>0){
 		err_status_t srtp_err;
 		/* keep NON-RTP data unencrypted */
@@ -85,9 +86,9 @@ static int  srtcp_sendto(RtpTransport *t, mblk_t *m, int flags, const struct soc
 	srtp_t srtp=(srtp_t)t->data;
 	int slen;
 	err_status_t srtp_err;
+	slen=msgdsize(m);
 	/* enlarge the buffer for srtp to write its data */
-	msgpullup(m,msgdsize(m)+SRTP_PAD_BYTES);
-	slen=m->b_wptr-m->b_rptr;
+	msgpullup(m,slen+SRTP_PAD_BYTES);
 	srtp_err=srtp_protect_rtcp(srtp,m->b_rptr,&slen);
 	if (srtp_err==err_status_ok){
 		return sendto(t->session->rtcp.socket,(void*)m->b_rptr,slen,flags,to,tolen);
@@ -100,9 +101,20 @@ static int srtcp_recvfrom(RtpTransport *t, mblk_t *m, int flags, struct sockaddr
 	srtp_t srtp=(srtp_t)t->data;
 	int err;
 	int slen;
-	err=recvfrom(t->session->rtcp.socket,(void*)m->b_wptr,m->b_datap->db_lim-m->b_datap->db_base,flags,from,fromlen);
+	err=rtp_session_rtp_recv_abstract(t->session->rtcp.socket,m,flags,from,fromlen);
 	if (err>0){
 		err_status_t srtp_err;
+		/* keep NON-RTP data unencrypted */
+		rtcp_common_header_t *rtcp;
+		if (err>=RTCP_COMMON_HEADER_SIZE)
+		{
+			rtcp = (rtcp_common_header_t*)m->b_wptr;
+			if (rtcp->version!=2)
+			{
+				return err;
+			}
+		}
+
 		slen=err;
 		srtp_err=srtp_unprotect_rtcp(srtp,m->b_wptr,&slen);
 		if (srtp_err==err_status_ok)
@@ -153,10 +165,32 @@ int srtp_transport_new(srtp_t srtp, RtpTransport **rtpt, RtpTransport **rtcpt ){
 	return 0;
 }
 
+static int srtp_init_done=0;
+
 err_status_t ortp_srtp_init(void)
 {
+	
+	err_status_t st=0;
 	ortp_message("srtp init");
-	return srtp_init();
+	if (!srtp_init_done) {
+		st=srtp_init();
+		if (st==0) {
+			srtp_init_done++;
+		}else{
+			ortp_fatal("Couldn't initialize SRTP library.");
+			err_reporting_init("oRTP");
+		}
+	}else srtp_init_done++;
+	return st;
+}
+
+void ortp_srtp_shutdown(void){
+	srtp_init_done--;
+	if (srtp_init_done==0){
+#ifdef HAVE_SRTP_SHUTDOWN
+		srtp_shutdown();
+#endif
+	}
 }
 
 err_status_t ortp_srtp_create(srtp_t *session, const srtp_policy_t *policy)
@@ -214,10 +248,10 @@ static bool_t ortp_init_srtp_policy(srtp_t srtp, srtp_policy_t* policy, enum ort
 			policy->rtp.cipher_key_len);
 			return FALSE;
 	}
-	key = (uint8_t*) malloc(key_size);
-	if (b64_decode((const char*)b64_key, b64_key_length, key, key_size) != key_size) {
+	key = (uint8_t*) ortp_malloc0(key_size+2); /*srtp uses padding*/
+	if (b64_decode(b64_key, b64_key_length, key, key_size) != key_size) {
 		ortp_error("Error decoding key");
-		free(key);
+		ortp_free(key);
 		return FALSE;
 	}
 		
@@ -228,11 +262,11 @@ static bool_t ortp_init_srtp_policy(srtp_t srtp, srtp_policy_t* policy, enum ort
 	err = ortp_srtp_add_stream(srtp, policy);
 	if (err != err_status_ok) {
 		ortp_error("Failed to add incoming stream to srtp session (%d)", err);
-		free(key);
+		ortp_free(key);
 		return FALSE;
 	}
 		
-	free(key);
+	ortp_free(key);
 	return TRUE;
 }
 
@@ -287,7 +321,7 @@ srtp_t ortp_srtp_create_configure_session(enum ortp_srtp_crypto_suite_t suite, u
 #else
 
 err_status_t ortp_srtp_init(void) {
-	return 0;
+	return -1;
 }
 
 err_status_t ortp_crypto_get_random(uint8_t *tmp, int size)
@@ -324,5 +358,8 @@ srtp_t ortp_srtp_create_configure_session(enum ortp_srtp_crypto_suite_t suite, u
 	return NULL;
 }
 
+void ortp_srtp_shutdown(void){
+}
+
 #endif
 
