@@ -24,8 +24,9 @@
 #elif HAVE_CONFIG_H
 #include "ortp-config.h"
 #endif
+#include "ortp/logging.h"
 #include "ortp/port.h"
-#include "ortp/ortp.h"
+#include "ortp/str_utils.h"
 #include "utils.h"
 
 #if	defined(_WIN32) && !defined(_WIN32_WCE)
@@ -170,7 +171,7 @@ int ortp_file_exist(const char *pathname) {
 char *ortp_strndup(const char *str,int n){
 	int min=MIN((int)strlen(str),n)+1;
 	char *ret=(char*)ortp_malloc(min);
-	strncpy(ret,str,n);
+	strncpy(ret,str,min);
 	ret[min-1]='\0';
 	return ret;
 }
@@ -201,25 +202,39 @@ int __ortp_thread_create(pthread_t *thread, pthread_attr_t *attr, void * (*routi
 
 int WIN_mutex_init(ortp_mutex_t *mutex, void *attr)
 {
+#ifdef WINAPI_FAMILY_PHONE_APP
+	InitializeSRWLock(mutex);
+#else
 	*mutex=CreateMutex(NULL, FALSE, NULL);
+#endif
 	return 0;
 }
 
 int WIN_mutex_lock(ortp_mutex_t * hMutex)
 {
+#ifdef WINAPI_FAMILY_PHONE_APP
+	AcquireSRWLockExclusive(hMutex);
+#else
 	WaitForSingleObject(*hMutex, INFINITE); /* == WAIT_TIMEOUT; */
+#endif
 	return 0;
 }
 
 int WIN_mutex_unlock(ortp_mutex_t * hMutex)
 {
+#ifdef WINAPI_FAMILY_PHONE_APP
+	ReleaseSRWLockExclusive(hMutex);
+#else
 	ReleaseMutex(*hMutex);
+#endif
 	return 0;
 }
 
 int WIN_mutex_destroy(ortp_mutex_t * hMutex)
 {
+#ifndef WINAPI_FAMILY_PHONE_APP
 	CloseHandle(*hMutex);
+#endif
 	return 0;
 }
 
@@ -253,7 +268,7 @@ int WIN_thread_join(ortp_thread_t thread_h, void **unused)
 {
 	if (thread_h!=NULL)
 	{
-		WaitForSingleObject(thread_h, INFINITE);
+		WaitForSingleObjectEx(thread_h, INFINITE, FALSE);
 		CloseHandle(thread_h);
 	}
 	return 0;
@@ -261,22 +276,34 @@ int WIN_thread_join(ortp_thread_t thread_h, void **unused)
 
 int WIN_cond_init(ortp_cond_t *cond, void *attr)
 {
+#ifdef WINAPI_FAMILY_PHONE_APP
+	InitializeConditionVariable(cond);
+#else
 	*cond=CreateEvent(NULL, FALSE, FALSE, NULL);
+#endif
 	return 0;
 }
 
 int WIN_cond_wait(ortp_cond_t* hCond, ortp_mutex_t * hMutex)
 {
+#ifdef WINAPI_FAMILY_PHONE_APP
+	SleepConditionVariableSRW(hCond, hMutex, INFINITE, 0);
+#else
 	//gulp: this is not very atomic ! bug here ?
 	WIN_mutex_unlock(hMutex);
 	WaitForSingleObject(*hCond, INFINITE);
 	WIN_mutex_lock(hMutex);
+#endif
 	return 0;
 }
 
 int WIN_cond_signal(ortp_cond_t * hCond)
 {
+#ifdef WINAPI_FAMILY_PHONE_APP
+	WakeConditionVariable(hCond);
+#else
 	SetEvent(*hCond);
+#endif
 	return 0;
 }
 
@@ -288,7 +315,9 @@ int WIN_cond_broadcast(ortp_cond_t * hCond)
 
 int WIN_cond_destroy(ortp_cond_t * hCond)
 {
+#ifndef WINAPI_FAMILY_PHONE_APP
 	CloseHandle(*hCond);
+#endif
 	return 0;
 }
 
@@ -411,6 +440,14 @@ int ortp_server_pipe_close_client(ortp_socket_t client){
 }
 
 int ortp_server_pipe_close(ortp_socket_t spipe){
+	struct sockaddr_un sa;
+	socklen_t len=sizeof(sa);
+	int err;
+	/*this is to retrieve the name of the pipe, in order to unlink the file*/
+	err=getsockname(spipe,(struct sockaddr*)&sa,&len);
+	if (err==0){
+		unlink(sa.sun_path);
+	}else ortp_error("getsockname(): %s",strerror(errno));
 	return close(spipe);
 }
 
@@ -462,15 +499,6 @@ void ortp_shm_close(void *mem){
 	shmdt(mem);
 }
 
-#else
-void *ortp_shm_open(unsigned int keyid, int size, int create){
-	ortp_error("No shared memory support for this OS.");
-}
-
-void ortp_shm_close(void *mem){
-}
-
-
 #endif
 
 #elif defined(WIN32) && !defined(_WIN32_WCE)
@@ -483,6 +511,10 @@ static HANDLE event=NULL;
 
 /* portable named pipes */
 ortp_pipe_t ortp_server_pipe_create(const char *name){
+#ifdef WINAPI_FAMILY_PHONE_APP
+	ortp_error("%s not supported!", __FUNCTION__);
+	return INVALID_HANDLE_VALUE;
+#else
 	ortp_pipe_t h;
 	char *pipename=make_pipe_name(name);
 	h=CreateNamedPipe(pipename,PIPE_ACCESS_DUPLEX|FILE_FLAG_OVERLAPPED,PIPE_TYPE_MESSAGE|PIPE_WAIT,1,
@@ -493,6 +525,7 @@ ortp_pipe_t ortp_server_pipe_create(const char *name){
 	}
 	if (event==NULL) event=CreateEvent(NULL,TRUE,FALSE,NULL);
 	return h;
+#endif
 }
 
 
@@ -501,6 +534,10 @@ even if nobody connects to the pipe.
 ortp_server_pipe_close() makes this function to exit.
 */
 ortp_pipe_t ortp_server_pipe_accept_client(ortp_pipe_t server){
+#ifdef WINAPI_FAMILY_PHONE_APP
+	ortp_error("%s not supported!", __FUNCTION__);
+	return INVALID_HANDLE_VALUE;
+#else
 	OVERLAPPED ol;
 	DWORD undef;
 	HANDLE handles[2];
@@ -516,19 +553,34 @@ ortp_pipe_t ortp_server_pipe_accept_client(ortp_pipe_t server){
 	}
 	CloseHandle(ol.hEvent);
 	return INVALID_HANDLE_VALUE;
+#endif
 }
 
 int ortp_server_pipe_close_client(ortp_pipe_t server){
+#ifdef WINAPI_FAMILY_PHONE_APP
+	ortp_error("%s not supported!", __FUNCTION__);
+	return -1;
+#else
 	return DisconnectNamedPipe(server)==TRUE ? 0 : -1;
+#endif
 }
 
 int ortp_server_pipe_close(ortp_pipe_t spipe){
+#ifdef WINAPI_FAMILY_PHONE_APP
+	ortp_error("%s not supported!", __FUNCTION__);
+	return -1;
+#else
 	SetEvent(event);
 	//CancelIoEx(spipe,NULL); /*vista only*/
 	return CloseHandle(spipe);
+#endif
 }
 
 ortp_pipe_t ortp_client_pipe_connect(const char *name){
+#ifdef WINAPI_FAMILY_PHONE_APP
+	ortp_error("%s not supported!", __FUNCTION__);
+	return INVALID_HANDLE_VALUE;
+#else
 	char *pipename=make_pipe_name(name);
 	ortp_pipe_t hpipe = CreateFile(
          pipename,   // pipe name
@@ -541,6 +593,7 @@ ortp_pipe_t ortp_client_pipe_connect(const char *name){
          NULL);          // no template file
 	ortp_free(pipename);
 	return hpipe;
+#endif
 }
 
 int ortp_pipe_read(ortp_pipe_t p, uint8_t *buf, int len){
@@ -564,11 +617,142 @@ int ortp_client_pipe_close(ortp_pipe_t sock){
 	return CloseHandle(sock);
 }
 
-void *ortp_shm_open(unsigned int keyid, int size, int create{
+
+typedef struct MapInfo{
+	HANDLE h;
+	void *mem;
+}MapInfo;
+
+static OList *maplist=NULL;
+
+void *ortp_shm_open(unsigned int keyid, int size, int create){
+#ifdef WINAPI_FAMILY_PHONE_APP
+	ortp_error("%s not supported!", __FUNCTION__);
 	return NULL;
+#else
+	HANDLE h;
+	char name[64];
+	void *buf;
+
+	snprintf(name,sizeof(name),"%x",keyid);
+	if (create){
+		h = CreateFileMapping(
+			INVALID_HANDLE_VALUE,    // use paging file
+			NULL,                    // default security 
+			PAGE_READWRITE,          // read/write access
+			0,                       // maximum object size (high-order DWORD) 
+			size,                // maximum object size (low-order DWORD)  
+			name);                 // name of mapping object
+	}else{
+		h = OpenFileMapping(
+			FILE_MAP_ALL_ACCESS,   // read/write access
+			FALSE,                 // do not inherit the name
+			name);               // name of mapping object 
+	}
+	if (h==(HANDLE)-1) {
+		ortp_error("Fail to open file mapping (create=%i)",create);
+		return NULL;
+	}
+	buf = (LPTSTR) MapViewOfFile(h, // handle to map object
+		FILE_MAP_ALL_ACCESS,  // read/write permission
+		0,                    
+		0,                    
+		size);
+	if (buf!=NULL){
+		MapInfo *i=(MapInfo*)ortp_new(MapInfo,1);
+		i->h=h;
+		i->mem=buf;
+		maplist=o_list_append(maplist,i);
+	}else{
+		CloseHandle(h);
+		ortp_error("MapViewOfFile failed");
+	}
+	return buf;
+#endif
 }
 
 void ortp_shm_close(void *mem){
+#ifdef WINAPI_FAMILY_PHONE_APP
+	ortp_error("%s not supported!", __FUNCTION__);
+#else
+	OList *elem;
+	for(elem=maplist;elem!=NULL;elem=elem->next){
+		MapInfo *i=(MapInfo*)elem->data;
+		if (i->mem==mem){
+			CloseHandle(i->h);
+			UnmapViewOfFile(mem);
+			ortp_free(i);
+			maplist=o_list_remove_link(maplist,elem);
+			return;
+		}
+	}
+	ortp_error("No shared memory at %p was found.",mem);
+#endif
 }
 
+
+#endif
+
+
+#ifdef __MACH__
+#include <sys/types.h>
+#include <sys/timeb.h>
+#endif
+
+void ortp_get_cur_time(ortpTimeSpec *ret){
+#if defined(_WIN32_WCE) || defined(WIN32)
+#ifdef WINAPI_FAMILY_PHONE_APP
+	ULONGLONG timemillis = GetTickCount64();
+	ret->tv_sec = timemillis / 1000;
+	ret->tv_nsec = (timemillis % 1000) * 1000000LL;
+#else
+	DWORD timemillis;
+#	if defined(_WIN32_WCE)
+	timemillis=GetTickCount();
+#	else
+	timemillis=timeGetTime();
+#	endif
+	ret->tv_sec=timemillis/1000;
+	ret->tv_nsec=(timemillis%1000)*1000000LL;
+#endif
+#elif defined(__MACH__) && defined(__GNUC__) && (__GNUC__ >= 3)
+	struct timeval tv;
+	gettimeofday(&tv, NULL);
+	ret->tv_sec=tv.tv_sec;
+	ret->tv_nsec=tv.tv_usec*1000LL;
+#elif defined(__MACH__)
+	struct timeb time_val;
+
+	ftime (&time_val);
+	ret->tv_sec = time_val.time;
+	ret->tv_nsec = time_val.millitm * 1000000LL;
+#else
+	struct timespec ts;
+	if (clock_gettime(CLOCK_MONOTONIC,&ts)<0){
+		ortp_fatal("clock_gettime() doesn't work: %s",strerror(errno));
+	}
+	ret->tv_sec=ts.tv_sec;
+	ret->tv_nsec=ts.tv_nsec;
+#endif
+}
+
+#if defined(_WIN32) && !defined(_MSC_VER)
+char* strtok_r(char *str, const char *delim, char **nextp){
+    char *ret;
+
+    if (str == NULL){
+        str = *nextp;
+    }
+    str += strspn(str, delim);
+    if (*str == '\0'){
+        return NULL;
+    }
+    ret = str;
+    str += strcspn(str, delim);
+    if (*str){
+        *str++ = '\0';
+    }
+    *nextp = str;
+    return ret;
+}
 #endif
